@@ -17,13 +17,15 @@
 
 #include "chatwindow.h"
 
-/*
-  Dario Abatianni
-*/
+#include <qtimer.h>
+
 
 class QCheckBox;
 class QStringList;
 class QTimer;
+class QListView;
+class QListViewItem;
+class QPushButton;
 
 class KListView;
 class KLineEdit;
@@ -51,7 +53,7 @@ class ChannelListPanel : public ChatWindow
     public slots:
         void addToChannelList(const QString& channel,int users,const QString& topic);
 
-        virtual void appendInputText(const QString&);
+        virtual void appendInputText(const QString&, bool fromCursor);
         void setFilter(const QString& filter);
 
         void applyFilterClicked();
@@ -139,5 +141,7 @@ class ChannelListPanel : public ChatWindow
         KLineEdit* filterInput;
 
         QString filterText;
+
+        int m_oldSortColumn;
 };
 #endif
