@@ -6,25 +6,30 @@
 */
 
 /*
-  copyright: (C) 2004 by Peter Simonsson
-  email:     psn@linux.se
+  Copyright (C) 2005-2007 Peter Simonsson <psn@linux.se>
+  Copyright (C) 2006 Dario Abatianni <eisfuchs@tigress.com>
+  Copyright (C) 2006-2007 Eike Hein <hein@kde.org>
 */
 
+#include "channeloptionsdialog.h"
+#include "konversationapplication.h"
+#include "channeloptionsui.h"
+#include "channel.h"
+
 #include <qcheckbox.h>
+#include <qpushbutton.h>
+#include <qregexp.h>
+#include <qheader.h>
+#include <qtoolbutton.h>
 
 #include <klocale.h>
 #include <klistview.h>
 #include <ktextedit.h>
 #include <klineedit.h>
 #include <knuminput.h>
-#include <qpushbutton.h>
-#include <qregexp.h>
-#include <qheader.h>
+#include <klistviewsearchline.h>
+#include <kiconloader.h>
 
-#include "channeloptionsdialog.h"
-#include "konversationapplication.h"
-#include "channeloptionsui.h"
-#include "channel.h"
 
 namespace Konversation
 {
@@ -42,7 +47,9 @@ namespace Konversation
 
         // don't allow sorting. most recent topic is always first
         m_widget->topicHistoryList->setSortColumn(-1);
+        m_widget->clearButton->setIconSet(SmallIconSet("locationbar_erase"));
         m_widget->banList->setDefaultRenameAction(QListView::Accept);
+        m_widget->banListSearchLine->setListView(m_widget->banList);
         // hide column where the complete topic will be put in for convenience
         m_widget->topicHistoryList->hideColumn(2);
         // do not allow the user to resize the hidden column back into view
@@ -61,8 +68,9 @@ namespace Konversation
         connect(m_channel, SIGNAL(modesChanged()), this, SLOT(refreshModes()));
         connect(m_channel->getOwnChannelNick(), SIGNAL(channelNickChanged()), this, SLOT(refreshEnableModes()));
 
-        connect(this, SIGNAL(cancelClicked()), this, SLOT(hide()));
+        connect(this, SIGNAL(cancelClicked()), this, SLOT(cancelClicked()));
         connect(this, SIGNAL(okClicked()), this, SLOT(changeOptions()));
+        connect(this, SIGNAL(okClicked()), this, SLOT(okClicked()));
 
         connect(m_channel, SIGNAL(banAdded(const QString&)), this, SLOT(addBan(const QString&)));
         connect(m_channel, SIGNAL(banRemoved(const QString&)), this, SLOT(removeBan(const QString&)));
@@ -182,7 +190,7 @@ namespace Konversation
             m_widget->topicPreview->setText(item->text(2));
         else
             // clear topic preview
-            m_widget->topicPreview->clear();
+            m_widget->topicPreview->setText("");
     }
 
     void ChannelOptionsDialog::refreshEnableModes()
@@ -242,7 +250,7 @@ namespace Konversation
         m_widget->moderatedModeChBox->setChecked(false);
         m_widget->secretModeChBox->setChecked(false);
         m_widget->keyModeChBox->setChecked(false);
-        m_widget->keyModeEdit->clear();
+        m_widget->keyModeEdit->setText("");
 
         QListViewItem* item = m_widget->otherModesList->firstChild();
 
@@ -369,21 +377,13 @@ namespace Konversation
         QStringList banlist = m_channel->getBanList();
         m_widget->banList->clear();
 
-        for(QStringList::const_iterator it = banlist.fromLast(); it != banlist.end(); --it)
-        {
-            QDateTime date;
-            date.setTime_t((*it).section(' ', 2 ,2).toUInt());
-
-            new BanListViewItem(m_widget->banList, (*it).section(' ', 0, 0), (*it).section(' ', 1, 1).section('!', 0, 0), date.toString(Qt::LocalDate));
-        }
+        for (QStringList::const_iterator it = banlist.fromLast(); it != banlist.end(); --it)
+            addBan((*it));
     }
 
     void ChannelOptionsDialog::addBan(const QString& newban)
     {
-        QDateTime date;
-        date.setTime_t(newban.section(' ', 2 ,2).toUInt());
-
-        new BanListViewItem(m_widget->banList, newban.section(' ', 0, 0), newban.section(' ', 1, 1).section('!', 0, 0), date.toString(Qt::LocalDate));
+        new BanListViewItem(m_widget->banList, newban.section(' ', 0, 0), newban.section(' ', 1, 1).section('!', 0, 0), newban.section(' ', 2 ,2).toUInt());
     }
 
     void ChannelOptionsDialog::removeBan(const QString& ban)
@@ -397,7 +397,7 @@ namespace Konversation
         {
             if (!m_NewBan->text(0).isEmpty())
             {
-                m_channel->getServer()->requestBan(QStringList(m_NewBan->text(0)), m_channel->getName(), QString::null);
+                m_channel->getServer()->requestBan(QStringList(m_NewBan->text(0)), m_channel->getName(), QString());
             }
 
             // We will delete the item and let the addBan slot handle
@@ -418,7 +418,7 @@ namespace Konversation
 
             if (!new_edited->text(0).isEmpty())
             {
-                m_channel->getServer()->requestBan(QStringList(new_edited->text(0)), m_channel->getName(), QString::null);
+                m_channel->getServer()->requestBan(QStringList(new_edited->text(0)), m_channel->getName(), QString());
             }
 
             // We delete the existing item because it's possible the server may
@@ -439,8 +439,29 @@ namespace Konversation
     void ChannelOptionsDialog::removeBanClicked()
     {
         if (m_widget->banList->currentItem())
-        {
             m_channel->getServer()->requestUnban(m_widget->banList->currentItem()->text(0), m_channel->getName());
+    }
+
+    void ChannelOptionsDialog::cancelClicked()
+    {
+        if (m_widget->banList->renameLineEdit()->isShown())
+        {
+            QKeyEvent e(QEvent::KeyPress, Qt::Key_Escape, 27, Qt::NoButton);
+
+            KApplication::sendEvent(m_widget->banList->renameLineEdit(), &e);
+        }
+
+        topicBeingEdited(false);
+        hide();
+    }
+
+    void ChannelOptionsDialog::okClicked()
+    {
+        if (m_widget->banList->renameLineEdit()->isShown())
+        {
+            QKeyEvent e(QEvent::KeyPress, Qt::Key_Return, 13, Qt::NoButton);
+
+            KApplication::sendEvent(m_widget->banList->renameLineEdit(), &e);
         }
     }
 
@@ -458,14 +479,43 @@ namespace Konversation
         m_isNewBan = isNew;
     }
 
-    BanListViewItem::BanListViewItem ( QListView *parent, QString label1, QString label2, QString label3, QString label4, QString label5, QString label6, QString label7, QString label8 ) : KListViewItem(parent, label1, label2, label3, label4, label5, label6, label7, label8)
+    BanListViewItem::BanListViewItem (QListView *parent, const QString& label1, const QString& label2,
+        uint timestamp) : KListViewItem(parent, label1, label2)
     {
         m_isNewBan = 0;
+        m_timestamp.setTime_t(timestamp);
     }
 
-    BanListViewItem::BanListViewItem ( QListView *parent, bool isNew, QString label1, QString label2, QString label3, QString label4, QString label5, QString label6, QString label7, QString label8 ) : KListViewItem(parent, label1, label2, label3, label4, label5, label6, label7, label8)
+    BanListViewItem::BanListViewItem (QListView *parent, bool isNew, const QString& label1, const QString& label2,
+        uint timestamp) : KListViewItem(parent, label1, label2)
     {
         m_isNewBan = isNew;
+        m_timestamp.setTime_t(timestamp);
+    }
+
+    QString BanListViewItem::text(int column) const
+    {
+        if (column == 2)
+            return KGlobal::locale()->formatDateTime(m_timestamp, true, true);
+
+        return KListViewItem::text(column);
+    }
+
+    int BanListViewItem::compare(QListViewItem *i, int col, bool ascending) const
+    {
+        if (col == 2)
+        {
+            BanListViewItem* item = static_cast<BanListViewItem*>(i);
+
+            if (m_timestamp == item->timestamp())
+                return 0;
+            else if (m_timestamp < item->timestamp())
+                return -1;
+            else
+                return 1;
+        }
+
+        return KListViewItem::compare(i, col, ascending);
     }
 
     void BanListViewItem::startRename( int col )
