@@ -12,14 +12,16 @@
 #ifndef KONVERSATIONCHANNELOPTIONSDIALOG_H
 #define KONVERSATIONCHANNELOPTIONSDIALOG_H
 
-#include <kdialogbase.h>
+#include "channel.h"
 
 #include <qstringlist.h>
-#include "channel.h"
+
+#include <kdialogbase.h>
+#include <klistview.h>
+
 
 namespace Konversation
 {
-
     class ChannelOptionsUI;
 
     class ChannelOptionsDialog : public KDialogBase
@@ -32,6 +34,7 @@ namespace Konversation
             QString topic();
             QStringList modes();
 
+
         public slots:
             void refreshTopicHistory();
             void refreshAllowedChannelModes();
@@ -48,39 +51,51 @@ namespace Konversation
 
             void changeOptions();
 
+
         protected slots:
             void topicHistoryItemClicked(QListViewItem* item);
             void topicBeingEdited(bool state);
 
+            void cancelClicked();
+            void okClicked();
+
+
         protected:
             bool m_editingTopic;
             QListViewItem *m_NewBan;
 
+
         private:
             ChannelOptionsUI* m_widget;
             Channel *m_channel;
     };
 
+
     // This is needed to overcome two deficiencies in KListViewItem
-    // First there is no signal emited when a rename is canceled
+    // First there is no signal emitted when a rename is canceled
     // Second there is no way to get the old value of an item after a rename
     class BanListViewItem : public KListViewItem
     {
         public:
-            BanListViewItem( QListView *parent );
-            BanListViewItem( QListView *parent, bool isNew );
-            BanListViewItem ( QListView *parent, const QString& label1, const QString& label2 = QString::null,
-                const QString& label3 = QString::null);
-            BanListViewItem ( QListView *parent, bool isNew, const QString& label1,
-                const QString& label2 = QString::null, const QString& label3 = QString::null);
-            virtual void startRename( int col );
-            virtual QString getOldValue() { return m_oldValue; }
+            explicit BanListViewItem( QListView *parent );
+            BanListViewItem(QListView *parent, bool isNew);
+            BanListViewItem(QListView *parent, const QString& label1, const QString& label2 = QString(), uint timestamp = 0);
+            BanListViewItem (QListView *parent, bool isNew, const QString& label1, const QString& label2 = QString(), uint timestamp = 0);
+
+            QString getOldValue() { return m_oldValue; }
+            QDateTime timestamp() { return m_timestamp; }
+
+            virtual QString text(int column) const;
+            virtual int compare(QListViewItem *i, int col, bool ascending) const;
+            virtual void startRename(int col);
+
 
         protected:
-            virtual void cancelRename ( int col );
+            virtual void cancelRename(int col);
 
             QString m_oldValue;
             bool m_isNewBan;
+            QDateTime m_timestamp;
     };
 
 }
