@@ -6,29 +6,23 @@
 */
 
 /*
-  Base class for all chat panels
-  begin:     Fri Feb 1 2002
-  copyright: (C) 2002 by Dario Abatianni
-  email:     eisfuchs@tigress.com
+  Copyright (C) 2002 Dario Abatianni <eisfuchs@tigress.com>
+  Copyright (C) 2006-2008 Eike Hein <hein@kde.org>
 */
 
 #ifndef CHATWINDOW_H
 #define CHATWINDOW_H
 
-#include <qvbox.h>
-#include <qfile.h>
-
 #include "identity.h"
 #include "common.h"
 
-/*
-  @author Dario Abatianni
-*/
+#include <qvbox.h>
+#include <qfile.h>
+
 
 class IRCView;
 class Server;
 class KonversationMainWindow;
-class ViewContainer;
 
 class ChatWindow : public QVBox
 {
@@ -44,7 +38,7 @@ class ChatWindow : public QVBox
             Channel,
             Query,
             DccChat,
-            DccPanel,
+            DccTransferPanel,
             RawLog,
             Notice,
             SNotice,
@@ -69,7 +63,6 @@ class ChatWindow : public QVBox
          *  @return The server it is associated with, or null if none.
          */
         Server* getServer();
-        virtual void setIdentity(const Identity *newIdentity);
         void setTextView(IRCView* newView);
         IRCView* getTextView() const;
         void setLog(bool activate);
@@ -79,11 +72,10 @@ class ChatWindow : public QVBox
         void setType(WindowType newType);
         WindowType getType();
 
-        virtual void insertRememberLine();
         virtual void append(const QString& nickname,const QString& message);
         virtual void appendRaw(const QString& message, bool suppressTimestamps=false);
-        virtual void appendQuery(const QString& nickname,const QString& message, bool usenotifications = false);
-        virtual void appendAction(const QString& nickname,const QString& message, bool usenotifications = false);
+        virtual void appendQuery(const QString& nickname,const QString& message, bool inChannel = false);
+        virtual void appendAction(const QString& nickname,const QString& message);
         virtual void appendServerMessage(const QString& type,const QString& message, bool parseURL = true);
         virtual void appendCommandMessage(const QString& command, const QString& message, bool important = true,
             bool parseURL = true, bool self = false);
@@ -93,7 +85,7 @@ class ChatWindow : public QVBox
 
         virtual QString getTextInLine();
         /** Clean up and close this tab.  Return false if you want to cancel the close. */
-        virtual bool closeYourself();
+        virtual bool closeYourself(bool askForConfirmation = true);
         /** Reimplement this to return true in all classes that /can/ become front view.
          */
         virtual bool canBeFrontView();
@@ -133,7 +125,7 @@ class ChatWindow : public QVBox
 
         signals:
         void nameChanged(ChatWindow* view, const QString& newName);
-        void online(ChatWindow* myself, bool state);
+        //void online(ChatWindow* myself, bool state);
         /** Emit this signal when you want to change the status bar text for this tab.
          *  It is ignored if this tab isn't focused.
          */
@@ -158,19 +150,18 @@ class ChatWindow : public QVBox
          */
         void adjustFocus();
 
-        virtual void appendInputText(const QString&);
+        virtual void appendInputText(const QString&, bool fromCursor);
         virtual void indicateAway(bool away);
 
+
         virtual void setNotificationsEnabled(bool enable) { m_notificationsEnabled = enable; }
+        void activateTabNotification(Konversation::TabNotifyType type);
         void resetTabNotification();
 
     protected slots:
         ///Used to disable functions when not connected
         virtual void serverOnline(bool online);
 
-        ///Checks if we should update tab notification or not
-        void activateTabNotification(Konversation::TabNotifyType type);
-
     protected:
 
         /** Some children may handle the name themselves, and not want this public.
@@ -202,7 +193,6 @@ class ChatWindow : public QVBox
          *  Not always non-null - e.g. for konsolepanel
          */
         Server* m_server;
-        Identity identity;
         QFile logfile;
         WindowType type;
 
