@@ -4,7 +4,7 @@
   copyright: (C) 2002 by Dario Abatianni
   email:     eisfuchs@tigress.com
 */
-// Copyright (C) 2004,2005 Shintaro Matsuoka <shin@shoegazed.org>
+// Copyright (C) 2004-2007 Shintaro Matsuoka <shin@shoegazed.org>
 // Copyright (C) 2004,2005 John Tapsell <john@geola.co.uk>
 
 /*
@@ -19,6 +19,9 @@
 
 #include "dcctransfer.h"
 
+#include <qfile.h>
+
+
 class QTimer;
 
 namespace KNetwork
@@ -27,41 +30,56 @@ namespace KNetwork
     class KStreamSocket;
 }
 
-class DccPanel;
-
 class DccTransferSend : public DccTransfer
 {
     Q_OBJECT
 
         public:
-        DccTransferSend( DccPanel* panel, const QString& partnerNick, const KURL& fileURL, const QString& ownIp, const QString &altFileName = QString::null, uint fileSize = -1);
+        DccTransferSend(QObject* parent);
         virtual ~DccTransferSend();
 
+        // REQUIRED
+        void setFileURL( const KURL& url );
+        // OPTIONAL
+        void setFileName( const QString& fileName );
+        // REQUIED
+        // FIXME: this setting should be an optional one or be removed: make DccTransferSend itself read the configuration
+        void setOwnIp( const QString& ownIp );
+        // OPTIONAL
+        void setFileSize( KIO::filesize_t fileSize );
+        // OPTIONAL
+        void setReverse( bool reverse );
+
         bool setResume( unsigned long position );
 
-        signals:
-        void sendReady( const QString& partner, const QString& fileName, const QString& ownIp, const QString& ownPort, unsigned long fileSize );
-
     public slots:
+        virtual bool queue();
         virtual void start();
         virtual void abort();
 
+        // invoked when the receiver accepts the offer (Reverse DCC)
+        void connectToReceiver( const QString& partnerHost, const QString& partnerPort );
+
     protected slots:
-        void heard();
+        void acceptClient();
+        // it must be invoked when m_sendSocket is ready
+        void startSending();
         void writeData();
         void getAck();
-        void socketError( int errorCode );
-        void connectionTimeout();
+        void slotGotSocketError( int errorCode );
+        void slotConnectionTimeout();
+        void slotConnectionFailed( int errorCode );
         void slotSendSocketClosed();
         void slotServerSocketClosed();
 
     protected:
         void cleanUp();
-        void failed(const QString& errorMessage = QString::null );
+        void failed(const QString& errorMessage = QString() );
+
         void startConnectionTimer( int sec );
         void stopConnectionTimer();
 
-        static QString getQFileErrorString( int code );
+        QString getQFileErrorString( int code );
 
         QFile m_file;
 
@@ -76,4 +94,5 @@ class DccTransferSend : public DccTransfer
 
         QTimer* m_connectionTimer;
 };
-#endif                                            // DCCTRANSFERSEND_H
+
+#endif  // DCCTRANSFERSEND_H
