@@ -8,21 +8,9 @@
 /*
   Copyright (C) 2002 Dario Abatianni <eisfuchs@tigress.com>
   Copyright (C) 2004 Peter Simonsson <psn@linux.se>
-  Copyright (C) 2006 Eike Hein <sho@eikehein.com>
+  Copyright (C) 2006-2008 Eike Hein <hein@kde.org>
 */
 
-#include <qdatastream.h>
-#include <qstringlist.h>
-#include <qdatetime.h>
-#include <qregexp.h>
-
-#include <klocale.h>
-#include <kdeversion.h>
-#include <kstringhandler.h>
-
-#include <config.h>
-#include <kdebug.h>
-
 #include "inputfilter.h"
 #include "server.h"
 #include "replycodes.h"
@@ -35,6 +23,16 @@
 #include "common.h"
 #include "notificationhandler.h"
 
+#include <qdatastream.h>
+#include <qstringlist.h>
+#include <qdatetime.h>
+#include <qregexp.h>
+
+#include <klocale.h>
+#include <kdeversion.h>
+#include <kstringhandler.h>
+#include <config.h>
+#include <kdebug.h>
 #include <kresolver.h>
 
 InputFilter::InputFilter()
@@ -150,9 +148,14 @@ void InputFilter::parseClientCommand(const QString &prefix, const QString &comma
                 {
                     Channel* channel = server->getChannelByName( parameterList[0] );
 
+                    if(!channel) {
+                        kdError() << "Didn't find the channel " << parameterList[0] << endl;
+                        return;
+                    }
+
                     channel->appendAction(sourceNick,ctcpArgument);
 
-                    if(channel && sourceNick != server->getNickname())
+                    if(sourceNick != server->getNickname())
                     {
                         if(ctcpArgument.lower().find(QRegExp("(^|[^\\d\\w])"
                             + QRegExp::escape(server->loweredNickname())
@@ -180,11 +183,11 @@ void InputFilter::parseClientCommand(const QString &prefix, const QString &comma
                     query = server->addQuery(nickinfo, false /* we didn't initiate this*/ );
 
                     // send action to query
-                    query->appendAction(sourceNick,ctcpArgument, true /*use notifications if enabled - e.g. OSD */);
+                    query->appendAction(sourceNick,ctcpArgument);
 
                     if(sourceNick != server->getNickname() && query)
                     {
-                        konv_app->notificationHandler()->nick(query, sourceNick, ctcpArgument);
+                        konv_app->notificationHandler()->queryMessage(query, sourceNick, ctcpArgument);
                     }
                 }
             }
@@ -240,7 +243,7 @@ void InputFilter::parseClientCommand(const QString &prefix, const QString &comma
                     else
                     {
                         // Do not internationalize the below version string
-                        reply = QString("Konversation %1 (C) 2002-2006 by the Konversation team")
+                        reply = QString("Konversation %1 (C) 2002-2008 by the Konversation team")
                             .arg(QString(KONVI_VERSION));
                     }
                     server->ctcpReply(sourceNick,"VERSION "+reply);
@@ -269,26 +272,81 @@ void InputFilter::parseClientCommand(const QString &prefix, const QString &comma
                     }
                     dccArgumentList += QStringList::split(' ', dccArguments);
 
-                    // Incoming file?
                     if(dccType=="send")
                     {
-                        konv_app->notificationHandler()->dccIncoming(server->getStatusView(), sourceNick);
-                        emit addDccGet(sourceNick,dccArgumentList);
+                        if(dccArgumentList.count()==4)
+                        {
+                            // incoming file
+                            konv_app->notificationHandler()->dccIncoming(server->getStatusView(), sourceNick);
+                            emit addDccGet(sourceNick,dccArgumentList);
+                        }
+                        else if(dccArgumentList.count()==5)
+                        {
+                            if(dccArgumentList[2]=="0")
+                            {
+                                // incoming file (Reverse DCC)
+                                konv_app->notificationHandler()->dccIncoming(server->getStatusView(), sourceNick);
+                                emit addDccGet(sourceNick,dccArgumentList);
+                            }
+                            else
+                            {
+                                // the receiver accepted the offer for Reverse DCC
+                                emit startReverseDccSendTransfer(sourceNick,dccArgumentList);
+                            }
+                        }
+                        else
+                        {
+                            server->appendMessageToFrontmost(i18n("DCC"),
+                                i18n("Received invalid DCC SEND request from %1.")
+                                .arg(sourceNick)
+                                );
+                        }
                     }
-                    // Incoming file that shall be resumed?
                     else if(dccType=="accept")
                     {
-                        emit resumeDccGetTransfer(sourceNick,dccArgumentList);
+                        // resume request was accepted
+                        if(dccArgumentList.count()==3)
+                        {
+                            emit resumeDccGetTransfer(sourceNick,dccArgumentList);
+                        }
+                        else
+                        {
+                            server->appendMessageToFrontmost(i18n("DCC"),
+                                i18n("Received invalid DCC ACCEPT request from %1.")
+                                .arg(sourceNick)
+                                );
+                        }
                     }
                     // Remote client wants our sent file resumed
                     else if(dccType=="resume")
                     {
-                        emit resumeDccSendTransfer(sourceNick,dccArgumentList);
+                        if(dccArgumentList.count()==3)
+                        {
+                            emit resumeDccSendTransfer(sourceNick,dccArgumentList);
+                        }
+                        else
+                        {
+                            server->appendMessageToFrontmost(i18n("DCC"),
+                                i18n("Received invalid DCC RESUME request from %1.")
+                                .arg(sourceNick)
+                                );
+                        }
                     }
                     else if(dccType=="chat")
                     {
-                        // will be connected via Server to KonversationMainWindow::addDccChat()
-                        emit addDccChat(server->getNickname(),sourceNick,server->getNumericalIp(),dccArgumentList,false);
+
+                        if(dccArgumentList.count()==3)
+                        {
+                            // will be connected via Server to KonversationMainWindow::addDccChat()
+                            emit addDccChat(server->getNickname(),sourceNick,dccArgumentList,false);
+                        }
+                        else
+                        {
+                            server->appendMessageToFrontmost(i18n("DCC"),
+                                i18n("Received invalid DCC CHAT request from %1.")
+                                .arg(sourceNick)
+                                );
+                        }
                     }
                     else
                     {
@@ -325,12 +383,12 @@ void InputFilter::parseClientCommand(const QString &prefix, const QString &comma
                         server->appendServerMessageToChannel(
                             parameterList[0],
                             "CTCP",
-                            i18n("Received unknown CTCP-%1 request from %2 to Channel %3")
+                            i18n("Received unknown CTCP-%1 request from %2 to Channel %3.")
                             .arg(ctcp).arg(sourceNick).arg(parameterList[0])
                             );
                     else
                         server->appendMessageToFrontmost(i18n("CTCP"),
-                            i18n("Received unknown CTCP-%1 request from %2")
+                            i18n("Received unknown CTCP-%1 request from %2.")
                             .arg(ctcp).arg(sourceNick)
                             );
                 }
@@ -339,50 +397,7 @@ void InputFilter::parseClientCommand(const QString &prefix, const QString &comma
         // No CTCP, so it's an ordinary channel or query message
         else
         {
-            trailing=static_cast<KonversationApplication*>(kapp)->doAutoreplace(trailing,false);
-            if (isChan)
-            {
-                if(!isIgnore(prefix,Ignore::Channel))
-                {
-                    Channel* channel = server->getChannelByName(parameterList[0]);
-                    if(channel)
-                    {
-                        channel->append(sourceNick, trailing);
-
-                        if(sourceNick != server->getNickname())
-                        {
-                            if(trailing.lower().find(QRegExp("(^|[^\\d\\w])" +
-                               QRegExp::escape(server->loweredNickname()) + "([^\\d\\w]|$)")) !=-1 )
-                            {
-                                konv_app->notificationHandler()->nick(channel, sourceNick, trailing);
-                            }
-                            else
-                            {
-                                konv_app->notificationHandler()->message(channel, sourceNick, trailing);
-                            }
-                        }
-                    }
-                }
-            }
-            else
-            {
-                if(!isIgnore(prefix,Ignore::Query))
-                {
-                    NickInfoPtr nickinfo = server->obtainNickInfo(sourceNick);
-                    nickinfo->setHostmask(sourceHostmask);
-
-                    // Create a new query (server will check for dupes)
-                    query = server->addQuery(nickinfo, false /*we didn't initiate this*/ );
-
-                    // send action to query
-                    query->appendQuery(sourceNick, trailing, true /*use notifications if enabled - e.g. OSD */ );
-
-                    if(sourceNick != server->getNickname() && query)
-                    {
-                        konv_app->notificationHandler()->nick(query, sourceNick, trailing);
-                    }
-                }
-            }
+            parsePrivMsg (prefix, parameterList, trailing);
         }
     }
     else if(command=="notice")
@@ -435,7 +450,7 @@ void InputFilter::parseClientCommand(const QString &prefix, const QString &comma
                     else
                     {
                         server->appendMessageToFrontmost(i18n("CTCP"),
-                            i18n("Received CTCP-%1 reply from %2: %3")
+                            i18n("Received CTCP-%1 reply from %2: %3.")
                             .arg(replyReason).arg(sourceNick).arg(reply)
                             );
                     }
@@ -498,12 +513,12 @@ void InputFilter::parseClientCommand(const QString &prefix, const QString &comma
             server->getChannelByName(channelName)->clearModeList();
 
             // Request modes for the channel
-            server->queue("MODE "+channelName);
+            server->queue("MODE "+channelName, Server::LowPriority);
 
             // Initiate channel ban list
             server->getChannelByName(channelName)->clearBanList();
             setAutomaticRequest("BANLIST",channelName,true);
-            server->queue("MODE "+channelName+" +b");
+            server->queue("MODE "+channelName+" +b", Server::LowPriority);
         }
         else
         {
@@ -572,7 +587,10 @@ void InputFilter::parseClientCommand(const QString &prefix, const QString &comma
     }
     else if(command=="mode") // mode #channel -/+ mmm params
     {
-        parseModes(sourceNick,parameterList);
+        QStringList params=parameterList;
+        if (!trailing.isEmpty())
+            params << trailing;
+        parseModes(sourceNick, params);
         Channel* channel = server->getChannelByName(parameterList[0]);
         if(sourceNick != server->getNickname())
         {
@@ -581,11 +599,18 @@ void InputFilter::parseClientCommand(const QString &prefix, const QString &comma
     }
     else if(command=="invite")
     {
+        QString channel;
+
+        if (parameterList.count() > 1)
+            channel = parameterList[1];
+        else
+            channel = trailing;
+
         server->appendMessageToFrontmost(i18n("Invite"),
-            i18n("%1 invited you to channel %2")
-            .arg(sourceNick).arg(trailing)
+            i18n("%1 invited you to channel %2.")
+            .arg(sourceNick).arg(channel)
             );
-        emit invitation(sourceNick,trailing);
+        emit invitation(sourceNick,channel);
     }
     else
     {
@@ -618,7 +643,8 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
             }
 
             // queue the reply to send it as soon as possible
-            server->queueAt(0,"PONG"+text);
+            server->queue("PONG"+text, Server::HighPriority);
+
         }
         else if(command=="error :closing link:")
         {
@@ -635,12 +661,23 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
         }
         else if(command=="mode")
         {
-            parseModes(prefix,parameterList);
+            QStringList params=parameterList;
+            if (!trailing.isEmpty())
+                params << trailing;
+            parseModes(prefix, params);
         }
         else if(command=="notice")
         {
             server->appendStatusMessage(i18n("Notice"),i18n("-%1- %2").arg(prefix).arg(trailing));
         }
+        else if(command=="kick")
+        {
+            server->nickWasKickedFromChannel(parameterList[0],parameterList[1],prefix,trailing);
+        }
+        else if(command == "privmsg")
+        {
+            parsePrivMsg(prefix, parameterList, trailing);
+        }
         // All yet unknown messages go into the frontmost window unaltered
         else
         {
@@ -666,9 +703,6 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
                     if(parameterList[0]!=server->getNickname())
                         server->renameNick(server->getNickname(), parameterList[0]);
 
-                    // Remember server's insternal name
-                    server->setIrcName(prefix);
-
                     // Send the welcome signal, so the server class knows we are connected properly
                     emit welcome(host);
                     m_connecting = true;
@@ -719,7 +753,7 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
                         pos = value.find(')',1);
                         if(pos==-1)
                         {
-                            server->setPrefixes (QString::null, value);
+                            server->setPrefixes(QString(), value);
                             // XXX if ) isn't in the string, NOTHING should be there. anyone got a server
                             if (value.length() || property.length())
                                 server->appendStatusMessage("","XXX Server sent bad PREFIX in RPL_ISUPPORT, please report.");
@@ -745,16 +779,22 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
                 }                                 // endfor
                 break;
             }
+            case RPL_UMODEIS:
+            {
+                QString message=QString("%1 %2").arg(i18n("Your personal modes are:")).arg(parameterList.join(" ").section(' ',1) + ' '+trailing);
+                server->appendMessageToFrontmost("Info", message);
+                break;
+            }
             case RPL_CHANNELMODEIS:
             {
                 const QString modeString=parameterList[2];
                 // This is the string the user will see
-                QString modesAre(QString::null);
+                QString modesAre;
                 QString message = i18n("Channel modes: ") + modeString;
 
                 for(unsigned int index=0;index<modeString.length();index++)
                 {
-                    QString parameter(QString::null);
+                    QString parameter;
                     int parameterCount=3;
                     char mode=modeString[index];
                     if(mode!='+')
@@ -797,9 +837,8 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
                         }
                         server->updateChannelModeWidgets(parameterList[1],mode,parameter);
                     }
-                }                                 // endfor
-                if(!modesAre.isEmpty())
-                    if (Preferences::useLiteralModes())
+                } // endfor
+                if(!modesAre.isEmpty() && Preferences::useLiteralModes())
                 {
                     server->appendCommandMessageToChannel(parameterList[1],i18n("Mode"),message);
                 }
@@ -811,6 +850,12 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
                 }
                 break;
             }
+            case RPL_CHANNELURLIS:
+            {// :niven.freenode.net 328 argonel #channel :http://www.buggeroff.com/
+                server->appendCommandMessageToChannel(parameterList[1], i18n("URL"),
+                    i18n("Channel URL: %1").arg(trailing));
+                break;
+            }
             case RPL_CHANNELCREATED:
             {
                 QDateTime when;
@@ -819,12 +864,21 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
                     i18n("This channel was created on %1.")
                     .arg(when.toString(Qt::LocalDate))
                     );
+
+                if(Preferences::autoWhoContinuousEnabled())
+                {
+                    emit endOfWho(parameterList[1]);
+                }
+
                 break;
             }
             case RPL_WHOISACCOUNT:
             {
-                server->appendMessageToFrontmost(i18n("Whois"),i18n("%1 is logged in as %2.").arg(parameterList[1]).arg(parameterList[2]));
-
+                // Display message only if this was not an automatic request.
+                if(getAutomaticRequest("WHOIS",parameterList[1])==0)
+                {
+                    server->appendMessageToFrontmost(i18n("Whois"),i18n("%1 is logged in as %2.").arg(parameterList[1]).arg(parameterList[2]));
+                }
                 break;
             }
             case RPL_NAMREPLY:
@@ -908,10 +962,8 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
                 {
                     server->appendCommandMessageToChannel(parameterList[1],i18n("Topic"),
                         i18n("The topic was set by %1 on %2.")
-                        .arg(parameterList[2]).arg(when.toString(Qt::LocalDate))
-                        );
-
-                    emit topicAuthor(parameterList[1],parameterList[2]);
+                        .arg(parameterList[2]).arg(when.toString(Qt::LocalDate)),
+                        false);
                 }
                 else
                 {
@@ -922,13 +974,17 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
                         );
                     setAutomaticRequest("TOPIC",parameterList[1],false);
                 }
+                emit topicAuthor(parameterList[1], parameterList[2], when);
 
                 break;
             }
             case RPL_WHOISACTUALLY:
             {
-                server->appendMessageToFrontmost(i18n("Whois"),i18n("%1 is actually using the host %2.").arg(parameterList[1]).arg(parameterList[2]));
-
+                // Display message only if this was not an automatic request.
+                if(getAutomaticRequest("WHOIS",parameterList[1])==0)
+                {
+                    server->appendMessageToFrontmost(i18n("Whois"),i18n("%1 is actually using the host %2.").arg(parameterList[1]).arg(parameterList[2]));
+                }
                 break;
             }
             case ERR_NOSUCHNICK:
@@ -939,7 +995,7 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
                 {
                     server->appendMessageToFrontmost(i18n("Error"),i18n("%1: No such nick/channel.").arg(parameterList[1]));
                 }
-                else
+                else if(getAutomaticRequest("WHOIS",parameterList[1])==0) //Display message only if this was not an automatic request.
                 {
                     server->appendMessageToFrontmost(i18n("Error"),i18n("No such nick: %1.").arg(parameterList[1]));
                     setAutomaticRequest("DNS", parameterList[1], false);
@@ -949,15 +1005,18 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
             }
             case ERR_NOSUCHCHANNEL:
             {
-                server->appendMessageToFrontmost(i18n("Error"),i18n("%1: No such channel.").arg(parameterList[1]));
-
+                // Display message only if this was not an automatic request.
+                if(getAutomaticRequest("WHOIS",parameterList[1])==0)
+                {
+                    server->appendMessageToFrontmost(i18n("Error"),i18n("%1: No such channel.").arg(parameterList[1]));
+                }
                 break;
             }
             // Nick already on the server, so try another one
             case ERR_NICKNAMEINUSE:
             {
                 // if we are already connected, don't try tro find another nick ourselves
-                if(server->connected())
+                if(server->isConnected())
                 {                                 // Show message
                     server->appendMessageToFrontmost(i18n("Nick"),i18n("Nickname already in use, try a different one."));
                 }
@@ -986,7 +1045,7 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
             }
             case ERR_ERRONEUSNICKNAME:
             {
-                if(server->connected())
+                if(server->isConnected())
                 {                                 // We are already connected. Just print the error message
                     server->appendMessageToFrontmost(i18n("Nick"), trailing);
                 }
@@ -1163,6 +1222,7 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
             }
             // From a WHOIS.
             //[19:11] :zahn.freenode.net 320 PhantomsDad psn :is an identified user
+            case RPL_WHOISIDENTIFY:
             case RPL_IDENTIFIED:
             {
                 NickInfo* nickInfo = server->getNickInfo(parameterList[1]);
@@ -1196,7 +1256,7 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
                     nickInfo->setAway(bAway);
                     if(!bAway)
                     {
-                        nickInfo->setAwayMessage(QString::null);
+                        nickInfo->setAwayMessage(QString());
                     }
                 }
                 // Display message only if this was not an automatic request.
@@ -1209,7 +1269,7 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
                             .arg(parameterList[2])
                             .arg(parameterList[3])
                             .arg(trailing.section(" ", 1))
-                            .arg(bAway?i18n(" (Away)"):QString::null)
+                            .arg(bAway?i18n(" (Away)"):QString())
                             , false); // Don't parse as url
                     }
                 }
@@ -1219,9 +1279,11 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
             {
                 if(!whoRequestList.isEmpty())
                 {                                 // for safety
-                    if(parameterList[1].lower()==whoRequestList.front())
+                    QStringList::iterator it = whoRequestList.find(parameterList[1].lower());
+
+                    if(it != whoRequestList.end())
                     {
-                        if(getAutomaticRequest("WHO",whoRequestList.front())==0)
+                        if(getAutomaticRequest("WHO", *it) == 0)
                         {
                             server->appendMessageToFrontmost(i18n("Who"),
                                 i18n("End of /WHO list for %1")
@@ -1229,9 +1291,10 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
                         }
                         else
                         {
-                            setAutomaticRequest("WHO",whoRequestList.front(),false);
+                            setAutomaticRequest("WHO", *it, false);
                         }
-                        whoRequestList.pop_front();
+
+                        whoRequestList.remove(it);
                     }
                     else
                     {
@@ -1248,6 +1311,7 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
                         << parameterList[1]
                         << endl;
                 }
+
                 emit endOfWho(parameterList[1]);
                 break;
             }
@@ -1315,7 +1379,7 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
                     if(voiceChannels.count())
                     {
                         server->appendMessageToFrontmost(i18n("Whois"),
-                            i18n("%1 has voice on channels:  %2")
+                            i18n("%1 has voice on channels: %2")
                             .arg(parameterList[1]).arg(voiceChannels.join(" "))
                             );
                     }
@@ -1360,7 +1424,7 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
                     // Clear the away state on assumption that if nick is away, this message will be followed
                     // by a 301 RPL_AWAY message.  Not necessary a invalid assumption, but what can we do?
                     nickInfo->setAway(false);
-                    nickInfo->setAwayMessage(QString::null);
+                    nickInfo->setAwayMessage(QString());
                 }
                 // Display message only if this was not an automatic request.
                 if(getAutomaticRequest("WHOIS",parameterList[1])==0)
@@ -1372,18 +1436,6 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
                 }
                 break;
             }
-            case RPL_WHOISIDENTIFY:
-            {
-                // Display message only if this was not an automatic request.
-                if(getAutomaticRequest("WHOIS",parameterList[1])==0)
-                {
-                    server->appendMessageToFrontmost(i18n("Whois"),
-                        i18n("%1 has identified for this nick.")
-                        .arg(parameterList[1])
-                        );
-                }
-                break;
-            }
             case RPL_WHOISHELPER:
             {
                 // Display message only if this was not an automatic request.
@@ -1401,10 +1453,10 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
                 // Display message only if this was not an automatic request.
                 if(getAutomaticRequest("WHOIS",parameterList[1])==0)
                 {
-                    server->appendMessageToFrontmost(i18n("Whois"),
-                        i18n("%1 is a network admin.")
-                        .arg(parameterList[1])
-                        );
+                    if (trailing.lower().simplifyWhiteSpace().startsWith("is an irc operator"))
+                        server->appendMessageToFrontmost(i18n("Whois"),i18n("%1 is an IRC Operator.").arg(parameterList[1]));
+                    else
+                        server->appendMessageToFrontmost(i18n("Whois"),QString("%1 %2").arg(parameterList[1]).arg(trailing));
                 }
                 break;
             }
@@ -1535,9 +1587,9 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
                         server->appendMessageToFrontmost(i18n("Userhost"),
                             i18n("%1 = nick, %2 = shows if nick is op, %3 = hostmask, %4 = shows away", "%1%2 is %3%4.")
                             .arg(nick)
-                            .arg((ircOp) ? i18n(" (IRC Operator)") : QString::null)
+                            .arg((ircOp) ? i18n(" (IRC Operator)") : QString())
                             .arg(mask)
-                            .arg((away) ? i18n(" (away)") : QString::null));
+                            .arg((away) ? i18n(" (away)") : QString()));
                     }
 
                     // was this an automatic request?
@@ -1550,7 +1602,7 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
             }
             case RPL_LISTSTART:                   //FIXME This reply is obsolete!!!
             {
-                if(getAutomaticRequest("LIST",QString::null)==0)
+                if(getAutomaticRequest("LIST",QString())==0)
                 {
                     server->appendMessageToFrontmost(i18n("List"),i18n("List of channels:"));
                 }
@@ -1558,7 +1610,7 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
             }
             case RPL_LIST:
             {
-                if(getAutomaticRequest("LIST",QString::null)==0)
+                if(getAutomaticRequest("LIST",QString())==0)
                 {
                     QString message;
                     message=i18n("%1 (%n user): %2", "%1 (%n users): %2", parameterList[2].toInt());
@@ -1574,13 +1626,13 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
             case RPL_LISTEND:
             {
                 // was this an automatic request?
-                if(getAutomaticRequest("LIST",QString::null)==0)
+                if(getAutomaticRequest("LIST",QString())==0)
                 {
                     server->appendMessageToFrontmost(i18n("List"),i18n("End of channel list."));
                 }
                 else
                 {
-                    setAutomaticRequest("LIST",QString::null,false);
+                    setAutomaticRequest("LIST",QString(),false);
                 }
                 break;
             }
@@ -1588,35 +1640,22 @@ void InputFilter::parseServerCommand(const QString &prefix, const QString &comma
             {
                 NickInfo* nickInfo = server->getNickInfo(parameterList[0]);
                 if (nickInfo) nickInfo->setAway(true);
-                server->appendMessageToFrontmost(i18n("Away"),i18n("You are now marked as being away."));
-                if (!server->isAway()) emit away();
+
+                server->setAway(true);
+
                 break;
             }
             case RPL_UNAWAY:
             {
                 NickInfo* nickInfo = server->getNickInfo(parameterList[0]);
-                if(nickInfo)
+
+                if (nickInfo)
                 {
                     nickInfo->setAway(false);
-                    nickInfo->setAwayMessage(QString::null);
+                    nickInfo->setAwayMessage(QString());
                 }
 
-                Identity identity = *(server->getIdentity());
-
-                if(server->isAway())
-                {
-                    if(identity.getShowAwayMessage())
-                    {
-                        QString message = identity.getReturnMessage();
-                        server->sendToAllChannels(message.replace(QRegExp("%t", false), server->awayTime()));
-                    }
-                    server->appendMessageToFrontmost(i18n("Away"),i18n("You are no longer marked as being away."));
-                    emit unAway();
-                }
-                else
-                {
-                    server->appendMessageToFrontmost(i18n("Away"),i18n("You are not marked as being away."));
-                }
+                server->setAway(false);
 
                 break;
             }
@@ -1725,6 +1764,26 @@ void InputFilter::parseModes(const QString &sourceNick, const QStringList &param
 {
     const QString modestring=parameterList[1];
 
+    if (!isAChannel(parameterList[0]))
+    {
+        QString message;
+        if (parameterList[0] == server->getNickname())
+        {
+            if (sourceNick == server->getNickname())
+            {
+                //XXX someone might care about the potentially unnecessary plural here
+                message = i18n("You have set personal modes: ") + modestring;
+            }
+            else
+            {   //XXX someone might care about the potentially unnecessary plural here
+                message = QString("%1 %2 %3").arg(sourceNick).arg(i18n("has changed your personal modes:")).arg(modestring);
+            }
+        }
+        if (!message.isEmpty())
+            server->appendStatusMessage(i18n("Mode"), message);
+        return;
+    }
+
     bool plus=false;
     int parameterIndex=0;
     // List of modes that need a parameter (note exception with -k and -l)
@@ -1835,6 +1894,91 @@ void InputFilter::setLagMeasuring(bool state) { lagMeasuring=state; }
 
 bool InputFilter::getLagMeasuring()           { return lagMeasuring; }
 
+void InputFilter::parsePrivMsg(const QString& prefix,
+                               const QStringList& parameterList,
+                               const QString& trailing)
+{
+    int pos = prefix.find("!");
+    QString source;
+    QString sourceHostmask;
+    QString message(trailing);
+
+    if(pos > 0)
+    {
+        source = prefix.left(pos);
+        sourceHostmask = prefix.mid(pos + 1);
+    }
+    else
+    {
+        source = prefix;
+    }
+
+    KonversationApplication* konv_app = static_cast<KonversationApplication*>(kapp);
+    message = konv_app->doAutoreplace(message, false);
+
+    if(isAChannel(parameterList[0]))
+    {
+        if(!isIgnore(prefix, Ignore::Channel))
+        {
+            Channel* channel = server->getChannelByName(parameterList[0]);
+            if(channel)
+            {
+                channel->append(source, message);
+
+                if(source != server->getNickname())
+                {
+                    QRegExp regexp("(^|[^\\d\\w])" +
+                            QRegExp::escape(server->loweredNickname()) +
+                            "([^\\d\\w]|$)");
+                    regexp.setCaseSensitive(false);
+                    if(message.find(regexp) !=-1 )
+                    {
+                        konv_app->notificationHandler()->nick(channel,
+                                source, message);
+                    }
+                    else
+                    {
+                        konv_app->notificationHandler()->message(channel,
+                                source, message);
+                    }
+                }
+            }
+        }
+    }
+    else
+    {
+        if(!isIgnore(prefix,Ignore::Query))
+        {
+            NickInfoPtr nickinfo = server->obtainNickInfo(source);
+            nickinfo->setHostmask(sourceHostmask);
+
+            // Create a new query (server will check for dupes)
+            query = server->addQuery(nickinfo, false /*we didn't initiate this*/ );
+
+            // send action to query
+            query->appendQuery(source, message);
+
+            if(source != server->getNickname() && query)
+            {
+                QRegExp regexp("(^|[^\\d\\w])" +
+                        QRegExp::escape(server->loweredNickname()) +
+                        "([^\\d\\w]|$)");
+                regexp.setCaseSensitive(false);
+                if(message.find(regexp) !=-1 )
+                {
+                    konv_app->notificationHandler()->nick(query,
+                            source, message);
+                }
+                else
+                {
+                    konv_app->notificationHandler()->queryMessage(query,
+                            source, message);
+                }
+            }
+        }
+    }
+}
+
 #include "inputfilter.moc"
 
 // kate: space-indent on; tab-width 4; indent-width 4; mixed-indent off; replace-tabs on;
