@@ -14,6 +14,7 @@
 
 #include <kdialogbase.h>
 
+
 class KCharSelect;
 class QChar;
 
@@ -23,10 +24,12 @@ namespace Konversation
     class InsertCharDialog : public KDialogBase
     {
         Q_OBJECT
-            public:
-            InsertCharDialog(const QString& font = QString::null, QWidget *parent = 0, const char *name = 0);
+
+        public:
+            explicit InsertCharDialog(const QString& font = QString(), QWidget *parent = 0, const char *name = 0);
             ~InsertCharDialog();
 
+            void setFont(const QFont &font);
             QChar chr();
 
         protected slots:
