@@ -9,18 +9,18 @@
 */
 
 /*
-  the text widget used for all text based panels
-  begin:     Sun Jan 20 2002
-  copyright: (C) 2002 by Dario Abatianni
-  email:     eisfuchs@tigress.com
+  Copyright (C) 2002 Dario Abatianni <eisfuchs@tigress.com>
+  Copyright (C) 2005-2006 Peter Simonsson <psn@linux.se>
+  Copyright (C) 2006-2008 Eike Hein <hein@kde.org>
 */
 
+#include "common.h"
+
 #include <qmap.h>
 #include <qfontdatabase.h>
 
 #include <ktextbrowser.h>
 
-#include "common.h"
 
 class QPixmap;
 class QStrList;
@@ -38,7 +38,7 @@ class IRCView : public KTextBrowser
 {
     Q_OBJECT
 
-        public:
+    public:
         IRCView(QWidget* parent,Server* newServer);
         ~IRCView();
 
@@ -49,7 +49,7 @@ class IRCView : public KTextBrowser
         // Returns the current nick under context menu.
         const QString& getContextNick() const;
 
-        void updateStyleSheet();
+        void enableParagraphSpacing();
 
         QPopupMenu* getPopup() const;
 
@@ -58,7 +58,8 @@ class IRCView : public KTextBrowser
             Copy,CopyUrl,SelectAll,
             Search,
             SendFile,
-            Bookmark
+            Bookmark,
+            SaveAs
         };
 
         void setChatWin(ChatWindow* chatWin);
@@ -72,9 +73,11 @@ class IRCView : public KTextBrowser
 
         void setNickAndChannelContextMenusEnabled(bool enable);
 
-        signals:
+        bool hasLines();
+
+
+    signals:
         // Notify container of new text and highlight state
-        void updateTabNotification(Konversation::TabNotifyType type);
         void gotFocus();                          // So we can set focus to input line
         void textToLog(const QString& text);
         void sendFile();
@@ -89,11 +92,16 @@ class IRCView : public KTextBrowser
         void clearStatusBarTempText();
 
     public slots:
+        void insertRememberLine();
+        void cancelRememberLine();
+        void insertMarkerLine();
+        void clearLines();
+
         void append(const QString& nick, const QString& message);
-        void appendLine();
         void appendRaw(const QString& message, bool suppressTimestamps=false, bool self = false);
-        void appendQuery(const QString& nick, const QString& message);
-        void appendAction(const QString& nick, const QString& message);
+        void appendQuery(const QString& nick, const QString& message, bool inChannel = false);
+        void appendQueryAction(const QString& nick, const QString& message);
+        void appendChannelAction(const QString& nick, const QString& message);
         void appendServerMessage(const QString& type, const QString& message, bool parseURL = true);
         void appendCommandMessage(const QString& command, const QString& message, bool important,
             bool parseURL=true, bool self=false);
@@ -113,14 +121,19 @@ class IRCView : public KTextBrowser
         // Updates the scrollbar position
         void updateScrollBarPos();
 
+	// Override ensureCursorVisible() to have an ability to disable this call
+	virtual void ensureCursorVisible();
+
     protected slots:
         void highlightedSlot(const QString& link);
+        void saveLinkAs(const QString& url);
 
     protected:
         void openLink(const QString &url, bool newTab=false);
         QString filter(const QString& line, const QString& defaultColor, const QString& who=NULL,
-            bool doHighlight=true, bool parseURL=true, bool self=false);
-        void doAppend(const QString& line, bool important=true, bool self=false);
+        bool doHighlight=true, bool parseURL=true, bool self=false);
+        void appendAction(const QString& nick, const QString& message);
+        void doAppend(const QString& line, bool self=false);
         void replaceDecoration(QString& line,char decoration,char replacement);
         virtual void contentsDragMoveEvent(QDragMoveEvent* e);
         virtual void contentsDropEvent(QDropEvent* e);
@@ -134,6 +147,7 @@ class IRCView : public KTextBrowser
 
         void hideEvent(QHideEvent* event);
         void showEvent(QShowEvent* event);
+        void paintEvent(QPaintEvent* event);
 
         bool contextMenu(QContextMenuEvent* ce);
 
@@ -148,13 +162,22 @@ class IRCView : public KTextBrowser
         QString timeStamp();
 
         /// Returns a formated nick string
-        QString createNickLine(const QString& nick, bool encapsulateNick = true);
+        QString createNickLine(const QString& nick, bool encapsulateNick = true, bool privMsg = false);
 
         // used by search function
         int m_findParagraph;
         int m_findIndex;
 
-        bool m_lastInsertionWasLine;
+        void appendLine(const QString& color);
+        void appendRememberLine();
+
+        void updateLineParagraphs(int numRemoved);
+        void wipeLineParagraphs();
+
+        int m_rememberLineParagraph;
+        bool m_rememberLineDirtyBit;
+
+        QValueList<int> m_markerLineParagraphs;
 
         // This is set to what we last sent status text to the statusbar.  Empty if we have sent clearStatusBarTempText() string
         QString m_lastStatusText;
@@ -170,6 +193,7 @@ class IRCView : public KTextBrowser
         Server* m_server;
         QPopupMenu* m_popup;
         int toggleMenuBarSeparator;
+        int copyUrlMenuSeparator;
         KPopupMenu* m_nickPopup;
         KPopupMenu* m_channelPopup;
         KPopupMenu* m_modes;
@@ -205,7 +229,14 @@ class IRCView : public KTextBrowser
         int m_channelPopupId;
 
         QFontDatabase m_fontDataBase;
+	
+	// Provide ability to disable parent ensureCursorVisible() call
+	bool m_disableEnsureCursorVisible;
+	// If this widget was painted at least once. Needed for scrollbar
+	// reset HACK as a precaution.
+	bool m_wasPainted;
 
         ChatWindow* m_chatWin;
+        friend class IRCStyleSheet;
 };
 #endif
