@@ -12,23 +12,24 @@
   email:     zipzippy@sonic.net
 */
 
-#include <kapplication.h>
-#include <kdebug.h>
-#include <dcopclient.h>
-
-#include <qstring.h>
-#include <klocale.h>
-
-#include "konversationapplication.h"
-#include "channel.h"
 #include "konvdcop.h"
+#include "konversationapplication.h"
+#include "connectionmanager.h"
+#include "awaymanager.h"
+#include "channel.h"
 #include "identity.h"
 #include "server.h"
 
-KonvDCOP::KonvDCOP()
-: DCOPObject("irc"),
-QObject(0,"irc")
+#include <kapplication.h>
+#include <kdebug.h>
+#include <dcopclient.h>
+#include <klocale.h>
+
+
+KonvDCOP::KonvDCOP() : DCOPObject("irc"), QObject(0, "irc")
 {
+    connectDCOPSignal("kdesktop", "KScreensaverIface", "KDE_start_screensaver()", "setScreenSaverStarted()", false);
+    connectDCOPSignal("kdesktop", "KScreensaverIface", "KDE_stop_screensaver()", "setScreenSaverStopped()", false);
 }
 
 void KonvDCOP::raw(const QString& server,const QString& command)
@@ -40,36 +41,50 @@ void KonvDCOP::raw(const QString& server,const QString& command)
 
 QStringList KonvDCOP::listServers()
 {
-    QStringList serverlist;
-    QPtrList<Server>servers = static_cast<KonversationApplication *>(kapp)->getServerList();
-    Server *server;
-    for ( server = servers.first(); server; server = servers.next() )
-        serverlist.append(server->getServerName());
-    return serverlist;
+    KonversationApplication* konvApp = static_cast<KonversationApplication*>(kapp);
+
+    QStringList hosts;
+    QPtrList<Server> serverList = konvApp->getConnectionManager()->getServerList();
+    Server* server;
+
+    for (server = serverList.first(); server; server = serverList.next())
+        if (server) hosts << server->getServerName();
+
+    return hosts;
 }
 
 QStringList KonvDCOP::listConnectedServers()
 {
-    QStringList serverlist;
-    QPtrList<Server>servers = static_cast<KonversationApplication *>(kapp)->getServerList();
-    Server *server;
-    for ( server = servers.first(); server; server = servers.next() )
-        if(server->connected())
-            serverlist.append(server->getServerName());
-    return serverlist;
+    KonversationApplication* konvApp = static_cast<KonversationApplication*>(kapp);
+
+    QStringList connectedHosts;
+    QPtrList<Server> serverList = konvApp->getConnectionManager()->getServerList();
+    Server* server;
+
+    for (server = serverList.first(); server; server = serverList.next())
+        if (server && server->isConnected()) connectedHosts << server->getServerName();
+
+    return connectedHosts;
 }
 
-void KonvDCOP::setAway(const QString &awaymessage)
+void KonvDCOP::setAway(const QString& awaymessage)
 {
-    if(awaymessage.isEmpty())
-        emit dcopMultiServerRaw("away " + i18n("Gone away for now."));
-    else
-        emit dcopMultiServerRaw("away " + awaymessage);
+    static_cast<KonversationApplication*>(kapp)->getAwayManager()->requestAllAway(awaymessage);
 }
 
 void KonvDCOP::setBack()
 {
-    emit dcopMultiServerRaw("away");
+    static_cast<KonversationApplication*>(kapp)->getAwayManager()->requestAllUnaway();
+}
+
+void KonvDCOP::setScreenSaverStarted()
+{
+    static_cast<KonversationApplication*>(kapp)->getAwayManager()->setManagedIdentitiesAway();
+}
+
+void KonvDCOP::setScreenSaverStopped()
+{
+    static_cast<KonversationApplication*>(kapp)->getAwayManager()->setManagedIdentitiesUnaway();
 }
 
 void KonvDCOP::sayToAll(const QString &message)
@@ -82,11 +97,26 @@ void KonvDCOP::actionToAll(const QString &message)
     emit dcopMultiServerRaw("me " + message);
 }
 
-void KonvDCOP::say(const QString& server,const QString& target,const QString& command)
+void KonvDCOP::say(const QString& _server,const QString& _target,const QString& _command)
 {
-    kdDebug() << "KonvDCOP::say()" << endl;
-    // Act as if the user typed it
-    emit dcopSay(server,target,command);
+    //Sadly, copy on write doesn't exist with QString::replace
+    QString server(_server), target(_target), command(_command);
+
+    // TODO: this just masks a greater problem - Server::addQuery will return a query for '' --argonel
+    // TODO: other DCOP calls need argument checking too --argonel
+    if (server.isEmpty() || target.isEmpty() || command.isEmpty())
+        kdDebug() <<  "KonvDCOP::say() requires 3 arguments." << endl;
+    else
+    {
+        command.replace('\n',"\\n");
+        command.replace('\r',"\\r");
+        target.remove('\n');
+        target.remove('\r');
+        server.remove('\n');
+        server.remove('\r');
+        // Act as if the user typed it
+        emit dcopSay(server,target,command);
+    }
 }
 
 void KonvDCOP::info(const QString& string)
@@ -107,32 +137,38 @@ void KonvDCOP::error(const QString& string)
     emit dcopInfo(QString("Error: %1").arg(string));
 }
 
-void KonvDCOP::insertRememberLine()
+void KonvDCOP::insertMarkerLine()
 {
-    emit dcopInsertRememberLine();
+    emit dcopInsertMarkerLine();
 }
 
-void KonvDCOP::connectToServer(const QString& url, int port, const QString& channel, const QString& password)
+void KonvDCOP::connectToServer(const QString& address, int port, const QString& channel, const QString& password)
 {
-    emit dcopConnectToServer(url, port, channel, password);
+    emit connectTo(Konversation::SilentlyReuseConnection, address, QString::number(port), password, "", channel);
 }
 
-QString KonvDCOP::getNickname (const QString &serverName)
+QString KonvDCOP::getNickname(const QString& serverName)
 {
-    Server* server = static_cast<KonversationApplication *>(kapp)->getServerByName(serverName);
-    if ( !server )
+    Server* server = KonversationApplication::instance()->getConnectionManager()->getServerByName(serverName);
+
+    if (!server)
     {
         error( i18n( "getNickname: Server %1 is not found." ).arg( serverName ) );
         return QString();
     }
+
     return server->getNickname();
 }
 
-QString KonvDCOP::getAnyNickname ()
+QString KonvDCOP::getAnyNickname()
 {
-    const QPtrList<Server>serverlist = static_cast<KonversationApplication *>(kapp)->getServerList();
-    Server *server = serverlist.getFirst();
-    return server->getNickname();
+    KonversationApplication* konvApp = static_cast<KonversationApplication*>(kapp);
+
+    Server* server = konvApp->getConnectionManager()->getAnyServer();
+
+    if (server) return server->getNickname();
+
+    return QString();
 }
 
 QString KonvDCOP::getChannelEncoding(const QString& server, const QString& channel)
@@ -150,8 +186,8 @@ QObject(0, "identity")
 QStringList KonvIdentDCOP::listIdentities()
 {
     QStringList identities;
-    QValueList<IdentityPtr> ids = Preferences::identityList();
-    for(QValueList<IdentityPtr>::iterator it = ids.begin(); it != ids.end(); ++it)
+    IdentityList ids = Preferences::identityList();
+    for(IdentityList::ConstIterator it = ids.begin(); it != ids.end(); ++it)
     {
         identities.append((*it)->getName());
     }
@@ -160,9 +196,9 @@ QStringList KonvIdentDCOP::listIdentities()
 
 void KonvIdentDCOP::setrealName(const QString &id_name, const QString& name)
 {
-    QValueList<IdentityPtr> ids = Preferences::identityList();
+    IdentityList ids = Preferences::identityList();
 
-    for(QValueList<IdentityPtr>::iterator it = ids.begin(); it != ids.end(); ++it)
+    for(IdentityList::iterator it = ids.begin(); it != ids.end(); ++it)
     {
         if ((*it)->getName() == id_name)
         {
@@ -175,9 +211,9 @@ void KonvIdentDCOP::setrealName(const QString &id_name, const QString& name)
 
 QString KonvIdentDCOP::getrealName(const QString &id_name)
 {
-    QValueList<IdentityPtr> ids = Preferences::identityList();
+    IdentityList ids = Preferences::identityList();
 
-    for(QValueList<IdentityPtr>::iterator it = ids.begin(); it != ids.end(); ++it)
+    for(IdentityList::ConstIterator it = ids.begin(); it != ids.end(); ++it)
     {
         if ((*it)->getName() == id_name)
         {
@@ -185,7 +221,7 @@ QString KonvIdentDCOP::getrealName(const QString &id_name)
         }
     }
 
-    return QString::null;
+    return QString();
 }
 
 void KonvIdentDCOP::setIdent(const QString &/*identity*/, const QString& /*ident*/)
@@ -246,6 +282,19 @@ QStringList KonvIdentDCOP::getNicknameList(const QString &identity)
     return Preferences::identityByName(identity)->getNicknameList();
 }
 
+void KonvIdentDCOP::setQuitReason(const QString &identity, const QString& reason)
+{
+    const Identity *i = Preferences::identityByName(identity);
+    const_cast<Identity *>(i)->setQuitReason(reason);
+    static_cast<KonversationApplication *>(kapp)->saveOptions(true);
+}
+
+QString KonvIdentDCOP::getQuitReason(const QString &identity)
+{
+    return Preferences::identityByName(identity)->getQuitReason();
+}
+
+
 void KonvIdentDCOP::setPartReason(const QString &identity, const QString& reason)
 {
     const Identity *i = Preferences::identityByName(identity);
