@@ -10,24 +10,14 @@
   Copyright (C) 2005 Ismail Donmez <ismail@kde.org>
   Copyright (C) 2005 Peter Simonsson <psn@linux.se>
   Copyright (C) 2005 John Tapsell <johnflux@gmail.com>
-  Copyright (C) 2005 Eike Hein <sho@eikehein.com>
+  Copyright (C) 2005-2008 Eike Hein <hein@kde.org>
 */
 
-#include <qtextcodec.h>
-#include <qregexp.h>
-#include <qfileinfo.h>
-
-#include <kdebug.h>
-#include <kconfig.h>
-#include <dcopclient.h>
-#include <kdeversion.h>
-#include <kstandarddirs.h>
-#include <klocale.h>
-#include <kmessagebox.h>
-#include <kiconloader.h>
-
 #include "konversationapplication.h"
 #include "konversationmainwindow.h"
+#include "connectionmanager.h"
+#include "awaymanager.h"
+#include "dcctransfermanager.h"
 #include "viewcontainer.h"
 #include "highlight.h"
 #include "server.h"
@@ -36,22 +26,40 @@
 #include "servergroupsettings.h"
 #include "serversettings.h"
 #include "channel.h"
-#include "nicklistview.h"
 #include "images.h"
 #include "notificationhandler.h"
 #include "commit.h"
 #include "version.h"
 
+#include <qtextcodec.h>
+#include <qregexp.h>
+#include <qfileinfo.h>
+
+#include <kdebug.h>
+#include <kcmdlineargs.h>
+#include <kconfig.h>
+#include <dcopclient.h>
+#include <kdeversion.h>
+#include <kstandarddirs.h>
+#include <klocale.h>
+#include <kmessagebox.h>
+#include <kiconloader.h>
+
+
 KonversationApplication::KonversationApplication()
 : KUniqueApplication(true, true, true)
 {
     mainWindow = 0;
+    m_connectionManager = 0;
+    m_awayManager = 0;
     quickConnectDialog = 0;
-    m_connectDelayed=false;
+    osd = 0;
 }
 
 KonversationApplication::~KonversationApplication()
 {
+    kdDebug() << k_funcinfo << endl;
+    Server::_stashRates();
     Preferences::writeConfig();
     saveOptions(false);
 
@@ -59,25 +67,32 @@ KonversationApplication::~KonversationApplication()
     delete dcopObject;
     //delete prefsDCOP;
     delete identDCOP;
-}
-
-void KonversationApplication::delayedConnectToServer(const QString& hostname, const QString& port, const QString& channel,
-const QString& nick, const QString& password,
-const bool& useSSL)
-{
-    m_hostName=hostname;
-    m_port=port;
-    m_channel=channel;
-    m_nick=nick;
-    m_password=password;
-    m_useSSL=useSSL;
-    m_connectDelayed=true;
+    delete osd;
+    osd = 0;
 }
 
 int KonversationApplication::newInstance()
 {
-    if(!mainWindow)
+    KCmdLineArgs* args = KCmdLineArgs::parsedArgs();
+    QCString url;
+    if (args->count() > 0) url = args->arg(0);
+
+    if (!mainWindow)
     {
+        connect(this, SIGNAL(shutDown()), this, SLOT(prepareShutdown()));
+
+        m_connectionManager = new ConnectionManager(this);
+
+        m_awayManager = new AwayManager(this);
+
+        connect(m_connectionManager, SIGNAL(identityOnline(int)), m_awayManager, SLOT(identityOnline(int)));
+        connect(m_connectionManager, SIGNAL(identityOffline(int)), m_awayManager, SLOT(identityOffline(int)));
+        connect(m_connectionManager, SIGNAL(identityOffline(int)), m_awayManager, SLOT(identityOffline(int)));
+        connect(m_connectionManager, SIGNAL(connectionChangedAwayState(bool)), m_awayManager, SLOT(updateGlobalAwayAction(bool)));
+
+        // an instance of DccTransferManager needs to be created before GUI class instances' creation.
+        m_dccTransferManager = new DccTransferManager(this);
+
         // make sure all vars are initialized properly
         quickConnectDialog = 0;
 
@@ -118,35 +133,44 @@ int KonversationApplication::newInstance()
 
         connect(mainWindow, SIGNAL(showQuickConnectDialog()), this, SLOT(openQuickConnectDialog()) );
         connect(Preferences::self(), SIGNAL(updateTrayIcon()), mainWindow, SLOT(updateTrayIcon()) );
-        connect(this, SIGNAL(prefsChanged()), mainWindow, SIGNAL(prefsChanged()) );
         connect(osd, SIGNAL(hidden()), mainWindow, SIGNAL(endNotification()));
         // take care of user style changes, setting back colors and stuff
 
         // apply GUI settings
         emit appearanceChanged();
-        mainWindow->show();
 
-        if (Preferences::showServerList()) mainWindow->openServerList();
+        if (Preferences::showTrayIcon() && (Preferences::hiddenToTray() || Preferences::hideToTrayOnStartup()))
+            mainWindow->hide();
+        else
+            mainWindow->show();
+
+        bool openServerList = Preferences::showServerList();
 
         // handle autoconnect on startup
         Konversation::ServerGroupList serverGroups = Preferences::serverGroupList();
 
-        if (!m_connectDelayed)
+        if (url.isEmpty() && !args->isSet("server"))
         {
             for (Konversation::ServerGroupList::iterator it = serverGroups.begin(); it != serverGroups.end(); ++it)
             {
-                if ((*it)->autoConnectEnabled()) connectToServer((*it)->id());
+                if ((*it)->autoConnectEnabled())
+                {
+                    openServerList = false;
+                    m_connectionManager->connectTo(Konversation::CreateNewConnection, (*it)->id());
+                }
             }
         }
-        else
-            quickConnectToServer(m_hostName, m_port, m_channel, m_nick, m_password, m_useSSL);
+
+        if (openServerList) mainWindow->openServerList();
+
+        connect(this, SIGNAL(serverGroupsChanged(const Konversation::ServerGroupSettings*)), this, SLOT(saveOptions()));
 
         // prepare dcop interface
         dcopObject = new KonvDCOP;
         kapp->dcopClient()->setDefaultObject(dcopObject->objId());
         identDCOP = new KonvIdentDCOP;
 
-        if(dcopObject)
+        if (dcopObject)
         {
             connect(dcopObject,SIGNAL (dcopMultiServerRaw(const QString&)),
                 this,SLOT (dcopMultiServerRaw(const QString&)) );
@@ -156,55 +180,56 @@ int KonversationApplication::newInstance()
                 this,SLOT (dcopSay(const QString&,const QString&,const QString&)) );
             connect(dcopObject,SIGNAL (dcopInfo(const QString&)),
                 this,SLOT (dcopInfo(const QString&)) );
-            connect(dcopObject,SIGNAL (dcopInsertRememberLine()),
-                mainWindow,SIGNAL(insertRememberLine()));
-            connect(dcopObject,SIGNAL(dcopConnectToServer(const QString&, int,const QString&, const QString&)),
-                this,SLOT(dcopConnectToServer(const QString&, int,const QString&, const QString&)));
+            connect(dcopObject,SIGNAL (dcopInsertMarkerLine()),
+                mainWindow,SIGNAL(insertMarkerLine()));
+            connect(dcopObject, SIGNAL(connectTo(Konversation::ConnectionFlag, const QString&, const QString&, const QString&, const QString&, const QString&, bool)),
+                m_connectionManager, SLOT(connectTo(Konversation::ConnectionFlag, const QString&, const QString&, const QString&, const QString&, const QString&, bool)));
         }
 
         m_notificationHandler = new Konversation::NotificationHandler(this);
     }
 
+    if (!url.isEmpty())
+        getConnectionManager()->connectTo(Konversation::SilentlyReuseConnection, url);
+    else if (args->isSet("server"))
+    {
+        getConnectionManager()->connectTo(Konversation::SilentlyReuseConnection,
+                                          args->getOption("server"),
+                                          args->getOption("port"),
+                                          args->getOption("password"),
+                                          args->getOption("nick"),
+                                          args->getOption("channel"),
+                                          args->isSet("ssl"));
+    }
+
     return KUniqueApplication::newInstance();
 }
 
-                                                  // static
 KonversationApplication* KonversationApplication::instance()
 {
-    return static_cast<KonversationApplication*>( KApplication::kApplication() );
+    return static_cast<KonversationApplication*>(KApplication::kApplication());
 }
 
-KonversationMainWindow *KonversationApplication::getMainWindow()
+void KonversationApplication::prepareShutdown()
 {
-    return mainWindow;
-}
-
-void KonversationApplication::toggleAway()
-{
-    bool anyservers = false;
-    bool alreadyaway = false;
-
-    Server* lookServer=serverList.first();
-
-    while(lookServer)
-    {
-        if(lookServer->isConnected())
-        {
-            anyservers = true;
-            if(lookServer->isAway())
-            {
-                alreadyaway= true;
-                break;
-            }
-        }
-        lookServer=serverList.next();
-    }
-
-    //alreadyaway is true if _any_ servers are away
-    if (alreadyaway)
-        sendMultiServerCommand("back", QString::null);
+    if (mainWindow->isHidden() && Preferences::showTrayIcon())
+        Preferences::setHiddenToTray(true);
     else
-        sendMultiServerCommand("away", QString::null);
+        Preferences::setHiddenToTray(false);
+
+    mainWindow->getViewContainer()->prepareShutdown();
+
+    m_awayManager->blockSignals(true);
+    delete m_awayManager;
+
+    m_connectionManager->quitServers();
+    m_connectionManager->blockSignals(true);
+    delete m_connectionManager;
+}
+
+void KonversationApplication::showQueueTuner(bool p)
+{
+    getMainWindow()->getViewContainer()->showQueueTuner(p);
 }
 
 void KonversationApplication::dcopMultiServerRaw(const QString &command)
@@ -212,41 +237,33 @@ void KonversationApplication::dcopMultiServerRaw(const QString &command)
     sendMultiServerCommand(command.section(' ', 0,0), command.section(' ', 1));
 }
 
-void KonversationApplication::dcopRaw(const QString& server, const QString &command)
+void KonversationApplication::dcopRaw(const QString& connection, const QString &command)
 {
-    Server* lookServer=serverList.first();
-    while(lookServer)
-    {
-        if(lookServer->getServerName()==server)
-        {
-            lookServer->dcopRaw(command);
-            //      break; // leave while loop
-            //FIXME:   <muesli> there's a reason for not breaking this loop...  [see comment for dcopSay]
-        }
-        lookServer=serverList.next();
-    }
+    Server* server = 0;
 
+    bool conversion = false;
+    int connectionId = connection.toInt(&conversion);
+
+    if (conversion) server = getConnectionManager()->getServerByConnectionId(connectionId);
+
+    if (!server) server = getConnectionManager()->getServerByName(connection);
+
+    if (server) server->dcopRaw(command);
 }
 
-void KonversationApplication::dcopSay(const QString& server,const QString& target,const QString& command)
+
+void KonversationApplication::dcopSay(const QString& connection, const QString& target, const QString& command)
 {
-    Server* lookServer=serverList.first();
-    while(lookServer)
-    {
-        if(lookServer->getServerName()==server)
-        {
-            lookServer->dcopSay(target,command);
-            //      break; // leave while loop
-            //FIXME:   <muesli> there's a reason for not breaking this loop, here (which would spent only some
-            //                  cpu cycles, anyways): I am connected to two bouncers at the same time, which are
-            //                  also named the same (same ip, no dns). if a dcopSay gets emerged, it will always
-            //                  get the _same_ server name as its parameter (both are named the same). although
-            //                  the channel it gets sent to, is on the second server, it will always try to send
-            //                  this information to a channel on the first server, which i didn't even join.
-            //                  this is def. a quick-fix, we should probably handle server-id's instead of -names.
-        }
-        lookServer=serverList.next();
-    }
+    Server* server = 0;
+
+    bool conversion = false;
+    int connectionId = connection.toInt(&conversion);
+
+    if (conversion) server = getConnectionManager()->getServerByConnectionId(connectionId);
+
+    if (!server) server = getConnectionManager()->getServerByName(connection);
+
+    if (server) server->dcopSay(target, command);
 }
 
 void KonversationApplication::dcopInfo(const QString& string)
@@ -254,238 +271,6 @@ void KonversationApplication::dcopInfo(const QString& string)
     mainWindow->getViewContainer()->appendToFrontmost(i18n("DCOP"), string, 0);
 }
 
-bool KonversationApplication::validateIdentity(IdentityPtr identity, bool interactive)
-{
-    QString errors;
-
-    if (identity->getIdent().isEmpty())
-        errors+=i18n("Please fill in your <b>Ident</b>.<br>");
-
-    if (identity->getRealName().isEmpty())
-        errors+=i18n("Please fill in your <b>Real name</b>.<br>");
-
-    if (identity->getNickname(0).isEmpty())
-        errors+=i18n("Please provide at least one <b>Nickname</b>.<br>");
-
-    if (!errors.isEmpty())
-    {
-        if (interactive)
-        {
-            int result = KMessageBox::warningContinueCancel(0,
-                            i18n("<qt>Your identity \"%1\" is not set up correctly:<br>%2</qt>")
-                                .arg(identity->getName()).arg(errors),
-                            i18n("Identity Settings"),
-                            i18n("Edit Identity..."));
-
-            if (result==KMessageBox::Continue)
-            {
-                identity = mainWindow->editIdentity(identity);
-
-                if (identity && validateIdentity(identity,false))
-                    return true;
-                else
-                    return false;
-            }
-            else
-            {
-                return false;
-            }
-        }
-
-        return false;
-    }
-
-    return true;
-}
-
-Server* KonversationApplication::connectToServerGroup(const QString& serverGroup)
-{
-    int serverGroupId = Preferences::serverGroupIdByName(serverGroup);
-
-    return connectToServer(serverGroupId);
-}
-
-Server* KonversationApplication::connectToServer(int serverGroupId, Konversation::ServerSettings quickServer)
-{
-    // Check if a server window with same name and port is already open
-    Server* lookServer = serverList.first();
-    Server* existingServer = 0;
-
-    while (lookServer)
-    {
-      if (lookServer->serverGroupSettings()->id() == serverGroupId)
-      {
-          existingServer = lookServer;
-          break;
-      }
-
-      lookServer = serverList.next();
-    }
-
-    // There's already a connection to this network.
-    if (existingServer)
-    {
-        // The connection is active, and the server list dialog requested
-        // to connect to a specific server.
-        if (existingServer->isConnected() && !quickServer.server().isEmpty())
-        {
-            // This server differs from the server we're presently connected to.
-            if (quickServer.server() != existingServer->getServerName())
-            {
-                int result = KMessageBox::warningContinueCancel(
-                mainWindow,
-                i18n("You are already connected to %1. Do you want to disconnect from '%2' and connect to '%3' instead?")
-                    .arg(existingServer->getServerGroup())
-                    .arg(existingServer->getServerName())
-                    .arg(quickServer.server()),
-                i18n("Already connected to %1").arg(existingServer->getServerGroup()),
-                i18n("Disconnect"),
-                "ReconnectDifferentServer");
-
-                // The user has chosen to connect to this different server
-                // instead of the current one.
-                if (result == KMessageBox::Continue)
-                {
-                    existingServer->disconnect();
-                    existingServer->serverGroupSettings()->clearQuickServerList();
-                    existingServer->serverGroupSettings()->setQuickServerList(quickServer);
-                    existingServer->resetCurrentServerIndex();
-                    existingServer->reconnect();
-                    return existingServer;
-                }
-                // The user wants to keep the current connection alive, so
-                // return the existing connection.
-                else
-                    return existingServer;
-            }
-            // We've gotten a request to connect to the same server we're
-            // already connected to, so return the existing connection.
-            else
-                return existingServer;
-        }
-        // We haven't been told to connect to a specific server and already have
-        // a connection to this network, so return it.
-        else if (existingServer->isConnected() && quickServer.server().isEmpty())
-        {
-            return existingServer;
-        }
-        // The connection is inactive.
-        else if (!existingServer->isConnected())
-        {
-            // We've been told to connect to a specific server. Do so.
-            if (!quickServer.server().isEmpty())
-            {
-                existingServer->serverGroupSettings()->clearQuickServerList();
-                existingServer->serverGroupSettings()->setQuickServerList(quickServer);
-                existingServer->resetCurrentServerIndex();
-                existingServer->reconnect();
-                return existingServer;
-            }
-            // No specific server was part of the request, so reconnect
-            // the old one.
-            else
-            {
-                existingServer->reconnect();
-                return existingServer;
-            }
-        }
-    }
-
-    Konversation::ServerGroupSettingsPtr serverGroup = Preferences::serverGroupById(serverGroupId);
-    IdentityPtr identity = serverGroup->identity();
-
-    if (!identity || !validateIdentity(identity))
-        return 0;
-
-    emit closeServerList();
-
-    bool clearQuickServerList = true;
-
-    // Have we been called with a quickServer? If so, append to list & prevent
-    // simple Server constructor from cleaing its quickServer list like it
-    // would do normally.
-    if (!quickServer.server().isEmpty())
-    {
-        serverGroup->setQuickServerList(quickServer);
-        clearQuickServerList = false;
-    }
-
-    mainWindow->show();
-
-    Server* newServer = new Server(mainWindow->getViewContainer(), serverGroupId, clearQuickServerList);
-
-    connect(mainWindow,SIGNAL (startNotifyTimer(int)),newServer,SLOT (startNotifyTimer(int)) );
-    connect(mainWindow,SIGNAL (quitServer()),newServer,SLOT (quitServer()) );
-    connect(newServer, SIGNAL(connectionChangedState(Server*, Server::State)),
-        mainWindow, SIGNAL(serverStateChanged(Server*, Server::State)));
-
-    connect(newServer,SIGNAL (nicksNowOnline(Server*,const QStringList&,bool)),mainWindow,SLOT (setOnlineList(Server*,const QStringList&,bool)) );
-
-    connect(newServer,SIGNAL (deleted(Server*)),this,SLOT (removeServer(Server*)) );
-
-    connect(newServer, SIGNAL(multiServerCommand(const QString&, const QString&)),
-        this, SLOT(sendMultiServerCommand(const QString&, const QString&)));
-    connect(newServer, SIGNAL(awayInsertRememberLine(Server*)), mainWindow, SIGNAL(insertRememberLine(Server*)));
-
-    serverList.append(newServer);
-
-    return newServer;
-}
-
-void KonversationApplication::quickConnectToServer(const QString& hostName, const QString& port, const QString& channel, const QString& nick, const QString& password, const bool& useSSL)
-{
-    //used for the quick connect dialog and /server command
-
-    IdentityPtr identity;
-    Konversation::ServerGroupSettingsPtr serverGroupOfServer;
-
-    // If server is in an existing group, use that group (first group if server is in multiple groups)
-    if (serverGroupOfServer = Preferences::serverGroupByServer(hostName))
-        identity = serverGroupOfServer->identity();
-    else
-        identity = Preferences::identityByName("Default");
-
-    if (!identity || !validateIdentity(identity))
-        return;
-
-    Server* newServer = new Server(mainWindow->getViewContainer(), hostName, port, channel, nick, password, useSSL);
-
-    connect(mainWindow,SIGNAL (startNotifyTimer(int)),newServer,SLOT (startNotifyTimer(int)) );
-    connect(mainWindow,SIGNAL (quitServer()),newServer,SLOT (quitServer()) );
-    connect(newServer, SIGNAL(connectionChangedState(Server*, Server::State)),
-        mainWindow, SIGNAL(serverStateChanged(Server*, Server::State)));
-
-    connect(newServer,SIGNAL (nicksNowOnline(Server*,const QStringList&,bool)),mainWindow,SLOT (setOnlineList(Server*,const QStringList&,bool)) );
-
-    connect(newServer,SIGNAL (deleted(Server*)),this,SLOT (removeServer(Server*)) );
-
-    connect(newServer, SIGNAL(multiServerCommand(const QString&, const QString&)),
-        this, SLOT(sendMultiServerCommand(const QString&, const QString&)));
-    connect(newServer, SIGNAL(awayInsertRememberLine(Server*)), mainWindow, SIGNAL(insertRememberLine(Server*)));
-
-    serverList.append(newServer);
-}
-
-Server* KonversationApplication::getServerByName(const QString& name)
-{
-    Server* lookServer=serverList.first();
-
-    while(lookServer)
-    {
-        if(lookServer->getServerName()==name) return lookServer;
-        lookServer=serverList.next();
-    }
-
-    return 0;
-}
-
-void KonversationApplication::removeServer(Server* server)
-{
-    serverList.setAutoDelete(false);              // don't delete items when they are removed
-    if(!serverList.remove(server))
-        kdDebug() << "Could not remove " << server->getServerName() << endl;
-}
-
 void KonversationApplication::readOptions()
 {
     // get standard config file
@@ -528,7 +313,11 @@ void KonversationApplication::readOptions()
             newIdentity->setShowAwayMessage(config->readBoolEntry("ShowAwayMessage"));
             newIdentity->setAwayMessage(config->readEntry("AwayMessage"));
             newIdentity->setReturnMessage(config->readEntry("ReturnMessage"));
+            newIdentity->setAutomaticAway(config->readBoolEntry("AutomaticAway", false));
+            newIdentity->setAwayInactivity(config->readNumEntry("AwayInactivity", 10));
+            newIdentity->setAutomaticUnaway(config->readBoolEntry("AutomaticUnaway", false));
 
+            newIdentity->setQuitReason(config->readEntry("QuitReason"));
             newIdentity->setPartReason(config->readEntry("PartReason"));
             newIdentity->setKickReason(config->readEntry("KickReason"));
 
@@ -596,7 +385,7 @@ void KonversationApplication::readOptions()
             for(it2 = tmp1.begin(); it2 != tmp1.end(); ++it2)
             {
                 config->setGroup((*it2));
-                server.setServer(config->readEntry("Server"));
+                server.setHost(config->readEntry("Server"));
                 server.setPort(config->readNumEntry("Port"));
                 server.setPassword(config->readEntry("Password"));
                 server.setSSLEnabled(config->readBoolEntry("SSLEnabled"));
@@ -696,7 +485,7 @@ void KonversationApplication::readOptions()
         unsigned int hiIndex;
         for(hiIndex=0;hiIndex<hiList.count();hiIndex+=2)
         {
-            Preferences::addHighlight(hiList[hiIndex],false,'#'+hiList[hiIndex+1],QString::null,QString::null);
+            Preferences::addHighlight(hiList[hiIndex],false,'#'+hiList[hiIndex+1],QString(),QString());
         }
 
         config->deleteEntry("Highlight");
@@ -734,12 +523,22 @@ void KonversationApplication::readOptions()
     if(!newList.isEmpty()) Preferences::setAliasList(newList);
 
     // Channel Encodings
-    QMap<QString,QString> channelEncodingsEntry=config->entryMap("Channel Encodings");
+    QMap<QString,QString> channelEncodingEntries=config->entryMap("Channel Encodings");
     QRegExp re("^(.+) ([^\\s]+)$");
-    QStringList channelEncodingsEntryKeys=channelEncodingsEntry.keys();
-    for(unsigned int i=0; i<channelEncodingsEntry.count(); ++i)
-        if(re.search(channelEncodingsEntryKeys[i]) > -1)
-            Preferences::setChannelEncoding(re.cap(1),re.cap(2),channelEncodingsEntry[channelEncodingsEntryKeys[i]]);
+    QStringList channelEncodingEntryKeys=channelEncodingEntries.keys();
+    QStringList::iterator itStr=channelEncodingEntryKeys.begin();
+    for(; itStr != channelEncodingEntryKeys.end(); ++itStr)
+    {
+        if(re.search(*itStr) > -1)
+        {
+            int serverGroupId = Preferences::serverGroupIdByName(re.cap(1));
+            if(serverGroupId != -1)
+                Preferences::setChannelEncoding(serverGroupId,re.cap(2),channelEncodingEntries[*itStr]);
+        }
+    }
+
+    // O, what a tangled web
+    Server::_fetchRates();
 }
 
 void KonversationApplication::saveOptions(bool updateGUI)
@@ -758,10 +557,10 @@ void KonversationApplication::saveOptions(bool updateGUI)
             config->deleteGroup(identities[index]);
     }
 
-    QValueList<IdentityPtr> identityList = Preferences::identityList();
+    IdentityList identityList = Preferences::identityList();
     int index = 0;
 
-    for(QValueList<IdentityPtr>::iterator it = identityList.begin(); it != identityList.end(); ++it)
+    for(IdentityList::ConstIterator it = identityList.begin(); it != identityList.end(); ++it)
     {
         IdentityPtr identity = (*it);
         config->setGroup(QString("Identity %1").arg(index));
@@ -776,6 +575,10 @@ void KonversationApplication::saveOptions(bool updateGUI)
         config->writeEntry("ShowAwayMessage",identity->getShowAwayMessage());
         config->writeEntry("AwayMessage",identity->getAwayMessage());
         config->writeEntry("ReturnMessage",identity->getReturnMessage());
+        config->writeEntry("AutomaticAway", identity->getAutomaticAway());
+        config->writeEntry("AwayInactivity", identity->getAwayInactivity());
+        config->writeEntry("AutomaticUnaway", identity->getAutomaticUnaway());
+        config->writeEntry("QuitReason",identity->getQuitReason());
         config->writeEntry("PartReason",identity->getPartReason());
         config->writeEntry("KickReason",identity->getKickReason());
         config->writeEntry("PreShellCommand",identity->getShellCommand());
@@ -784,9 +587,6 @@ void KonversationApplication::saveOptions(bool updateGUI)
         index++;
     }                                             // endfor
 
-    // FIXME: check if this group is still needed
-    config->setGroup("Notify List");
-
     // Remove the old servergroups from the config
     QStringList groups = config->groupList().grep(QRegExp("ServerGroup [0-9]+"));
     if(groups.count())
@@ -838,7 +638,7 @@ void KonversationApplication::saveOptions(bool updateGUI)
 
     for(it = serverGroupList.begin(); it != serverGroupList.end(); ++it)
     {
-        serverlist = (*it)->serverList(true);
+        serverlist = (*it)->serverList();
         servers.clear();
 
         for(it2 = serverlist.begin(); it2 != serverlist.end(); ++it2)
@@ -846,7 +646,7 @@ void KonversationApplication::saveOptions(bool updateGUI)
             groupName = QString("Server %1").arg(index2);
             servers.append(groupName);
             config->setGroup(groupName);
-            config->writeEntry("Server", (*it2).server());
+            config->writeEntry("Server", (*it2).host());
             config->writeEntry("Port", (*it2).port());
             config->writeEntry("Password", (*it2).password());
             config->writeEntry("SSLEnabled", (*it2).SSLEnabled());
@@ -895,31 +695,6 @@ void KonversationApplication::saveOptions(bool updateGUI)
     }
 
     config->deleteGroup("Server List");
-/*
-    Should be done in HighlightConfigController now ...
-    remove this part as soon as we are certain it works
-
-    // Write all highlight entries
-    QPtrList<Highlight> hiList=Preferences::highlightList();
-    int i = 0;
-    for(Highlight* hl = hiList.first(); hl; hl = hiList.next())
-    {
-        config->setGroup(QString("Highlight%1").arg(i));
-        config->writeEntry("Pattern", hl->getPattern());
-        config->writeEntry("RegExp", hl->getRegExp());
-        config->writeEntry("Color", hl->getColor());
-        config->writePathEntry("Sound", hl->getSoundURL().prettyURL());
-        config->writeEntry("AutoText", hl->getAutoText());
-        i++;
-    }
-
-    // Remove unused entries...
-    while(config->hasGroup(QString("Highlight%1").arg(i)))
-    {
-        config->deleteGroup(QString("Highlight%1").arg(i));
-        i++;
-    }
-*/
 
     // Ignore List
     config->deleteGroup("Ignore List");
@@ -935,16 +710,27 @@ void KonversationApplication::saveOptions(bool updateGUI)
     }
 
     // Channel Encodings
+    // remove all entries once
+    config->deleteGroup("Channel Encodings");
     config->setGroup("Channel Encodings");
-    QStringList channelEncodingsServerList=Preferences::channelEncodingsServerList();
-    channelEncodingsServerList.sort();
-    for(unsigned int i=0; i<channelEncodingsServerList.count(); ++i)
+    QValueList<int> encServers=Preferences::channelEncodingsServerGroupIdList();
+    //i have no idea these would need to be sorted //encServers.sort();
+    QValueList<int>::iterator encServer;
+    for ( encServer = encServers.begin(); encServer != encServers.end(); ++encServer )
     {
-        QStringList channelEncodingsChannelList=Preferences::channelEncodingsChannelList(channelEncodingsServerList[i]);
-        channelEncodingsChannelList.sort();
-        for(unsigned int j=0; j<channelEncodingsChannelList.count(); ++j)
-            if(!Preferences::channelEncoding(channelEncodingsServerList[i],channelEncodingsChannelList[j]).isEmpty())
-                config->writeEntry(channelEncodingsServerList[i]+' '+channelEncodingsChannelList[j],Preferences::channelEncoding(channelEncodingsServerList[i],channelEncodingsChannelList[j]));
+        Konversation::ServerGroupSettingsPtr sgsp = Preferences::serverGroupById(*encServer);
+        if ( sgsp )  // sgsp == 0 when the entry is of QuickConnect or something?
+        {
+            QStringList encChannels=Preferences::channelEncodingsChannelList(*encServer);
+            //ditto //encChannels.sort();
+            QStringList::iterator encChannel;
+            for ( encChannel = encChannels.begin(); encChannel != encChannels.end(); ++encChannel )
+            {
+                QString enc = Preferences::channelEncoding(*encServer, *encChannel);
+                QString key = sgsp->name() + ' ' + (*encChannel);
+                config->writeEntry(key, enc);
+            }
+        }
     }
 
     config->sync();
@@ -953,23 +739,6 @@ void KonversationApplication::saveOptions(bool updateGUI)
         emit appearanceChanged();
 }
 
-void KonversationApplication::updateNickIcons()
-{
-    Server* lookServer=serverList.first();
-
-    while(lookServer)
-    {
-        QPtrList<Channel> channelList = lookServer->getChannelList();
-        Channel* channel=channelList.first();
-        while(channel)
-        {
-            channel->getNickListView()->refresh();
-            channel=channelList.next();
-        }
-        lookServer=serverList.next();
-    }
-}
-
 // FIXME: use KURL maybe?
 void KonversationApplication::storeUrl(const QString& who,const QString& newUrl)
 {
@@ -1004,71 +773,21 @@ void KonversationApplication::clearUrlList()
 void KonversationApplication::openQuickConnectDialog()
 {
     quickConnectDialog = new QuickConnectDialog(mainWindow);
-    connect(quickConnectDialog, SIGNAL(connectClicked(const QString&, const QString&, const QString&, const QString&, const QString&, const bool&)),this, SLOT(quickConnectToServer(const QString&, const QString&, const QString&, const QString&, const QString&,const bool&)));
+    connect(quickConnectDialog, SIGNAL(connectClicked(Konversation::ConnectionFlag, const QString&, const QString&,
+        const QString&, const QString&, const QString&, bool)),
+        m_connectionManager, SLOT(connectTo(Konversation::ConnectionFlag, const QString&, const QString&,
+        const QString&, const QString&, const QString&, bool)));
     quickConnectDialog->show();
 }
 
-bool KonversationApplication::emitDCOPSig(const QString &appId, const QString &objId, const QString &signal, QByteArray &data)
-{
-    kdDebug() << "emitDCOPSig (" << signal << ")" << endl;
-    //dcopObject->emitDCOPSignal(signal, data);
-    QByteArray replyData;
-    QCString replyType;
-    if (!KApplication::dcopClient()->call(appId.ascii(), objId.ascii(), signal.ascii() /*must have prototype*/,
-        data, replyType, replyData))
-    {
-        kdDebug() << "There was some error using DCOP." << endl;
-        return true;                              // Keep processing filters
-    }
-    else
-    {
-        QDataStream reply(replyData, IO_ReadOnly);
-        if (replyType == "bool")
-        {
-            bool result;
-            reply >> result;
-            return result;
-        }
-        else
-        {
-            kdDebug() << "doIt returned an unexpected type of reply!" << endl;
-            return true;                          // Keep processing
-        }
-    }
-}
-
 void KonversationApplication::sendMultiServerCommand(const QString& command, const QString& parameter)
 {
-    for(Server* server = serverList.first(); server; server = serverList.next())
-    {
+    QPtrList<Server> serverList = getConnectionManager()->getServerList();
+
+    for (Server* server = serverList.first(); server; server = serverList.next())
         server->executeMultiServerCommand(command, parameter);
-    }
 }
 
-void KonversationApplication::dcopConnectToServer(const QString& url, int port, const QString& channel,
-const QString& password)
-{
-    Server* server = getServerByName(url);
-
-    if(server)
-        server->sendJoinCommand(channel);
-    else
-        quickConnectToServer(url, QString::number(port), channel, password);
-}
-
-Konversation::Sound* KonversationApplication::sound()
-{
-    return m_sound;
-}
-
-Images* KonversationApplication::images()
-{
-    return m_images;
-}
-
-// Returns list of pointers to Servers.
-const QPtrList<Server> KonversationApplication::getServerList() { return serverList; }
-
 void KonversationApplication::splitNick_Server(const QString& nick_server, QString &ircnick, QString &serverOrGroup)
 {
     //kaddresbook uses the utf separator 0xE120, so treat that as a separator as well
@@ -1080,13 +799,14 @@ void KonversationApplication::splitNick_Server(const QString& nick_server, QStri
 
 NickInfoPtr KonversationApplication::getNickInfo(const QString &ircnick, const QString &serverOrGroup)
 {
+    QPtrList<Server> serverList = getConnectionManager()->getServerList();
     NickInfoPtr nickInfo;
     QString lserverOrGroup = serverOrGroup.lower();
     for(Server* lookServer = serverList.first(); lookServer; lookServer = serverList.next())
     {
         if(lserverOrGroup.isEmpty()
             || lookServer->getServerName().lower()==lserverOrGroup
-            || lookServer->getServerGroup().lower()==lserverOrGroup)
+            || lookServer->getDisplayName().lower()==lserverOrGroup)
         {
             nickInfo = lookServer->getNickInfo(ircnick);
             if(nickInfo) return nickInfo;         //If we found one
@@ -1125,24 +845,30 @@ QString KonversationApplication::doAutoreplace(const QString& text,bool output)
                 // create regex from pattern
                 QRegExp needleReg=pattern;
                 // set pattern case insensitive
-                needleReg.setCaseSensitive(false);
-                // find matches
-                if(line.find(needleReg)!=-1)
-                {
-                    // prepare list of captured ( ) groups
-                    QStringList captures;
-                    // remember captured patterns
-                    captures=needleReg.capturedTexts();
+                needleReg.setCaseSensitive(true);
+                int index = 0;
 
-                    // replace %0 - %9 in regex groups
-                    for(unsigned int capture=0;capture<captures.count();capture++)
+                do {
+                    replacement = definition.section(',',3);
+                    // find matches
+                    index = line.find(needleReg, index);
+
+                    if(index != -1)
                     {
-                        replacement.replace(QString("%%1").arg(capture),captures[capture]);
+                        // remember captured patterns
+                        QStringList captures = needleReg.capturedTexts();
+
+                        // replace %0 - %9 in regex groups
+                        for(unsigned int capture=0;capture<captures.count();capture++)
+                        {
+                            replacement.replace(QString("%%1").arg(capture),captures[capture]);
+                        }
+                        replacement.replace(QRegExp("%[0-9]"),QString());
+                        // replace input with replacement
+                        line.replace(index, captures[0].length(), replacement);
+                        index += replacement.length();
                     }
-                    replacement.replace(QRegExp("%[0-9]"),QString::null);
-                    // replace input with replacement
-                    line.replace(needleReg,replacement);
-                }
+                } while(index >= 0 && index < (int)line.length());
             }
             else
             {
