@@ -10,9 +10,25 @@
   Copyright (C) 2005 Ismail Donmez <ismail@kde.org>
   Copyright (C) 2005 Peter Simonsson <psn@linux.se>
   Copyright (C) 2005 John Tapsell <johnflux@gmail.com>
-  Copyright (C) 2005 Eike Hein <sho@eikehein.com>
+  Copyright (C) 2005-2008 Eike Hein <hein@kde.org>
 */
 
+#include "konversationmainwindow.h"
+#include "konversationapplication.h"
+#include "linkaddressbook/addressbook.h"
+#include "konvisettingsdialog.h"
+#include "viewcontainer.h"
+#include "konversationstatusbar.h"
+#include "konvibookmarkhandler.h"
+#include "trayicon.h"
+#include "serverlistdialog.h"
+#include "identitydialog.h"
+#include "notificationhandler.h"
+#include "irccharsets.h"
+#include "connectionmanager.h"
+#include "awaymanager.h"
+#include "dcctransfermanager.h"
+
 #include <qnamespace.h>
 #include <qwhatsthis.h>
 #include <qsignalmapper.h>
@@ -43,19 +59,6 @@
 #include <knotifydialog.h>
 #endif
 
-#include "linkaddressbook/addressbook.h"
-#include "konversationmainwindow.h"
-#include "konvisettingsdialog.h"
-#include "viewcontainer.h"
-#include "konversationstatusbar.h"
-#include "konvibookmarkhandler.h"
-#include "konversationapplication.h"
-#include "trayicon.h"
-#include "serverlistdialog.h"
-#include "identitydialog.h"
-#include "notificationhandler.h"
-#include "irccharsets.h"
-#include "konviiphelper.h"
 
 KonversationMainWindow::KonversationMainWindow() : KMainWindow(0,"main_window", WStyle_ContextHelp | WType_TopLevel | WDestructiveClose)
 {
@@ -75,11 +78,18 @@ KonversationMainWindow::KonversationMainWindow() : KMainWindow(0,"main_window",
     // Set up view container
     connect(KonversationApplication::instance(), SIGNAL(appearanceChanged()), m_viewContainer, SLOT(updateAppearance()));
     connect(KonversationApplication::instance(), SIGNAL(iconChanged(int)), m_viewContainer, SLOT(updateViewIcons()));
+    connect(KonversationApplication::instance(), SIGNAL(serverGroupsChanged(const Konversation::ServerGroupSettings*)),
+            m_viewContainer, SLOT(updateViews(const Konversation::ServerGroupSettings*)));
+    connect(m_viewContainer, SIGNAL(autoJoinToggled(const Konversation::ServerGroupSettings*)),
+            KonversationApplication::instance(), SIGNAL(serverGroupsChanged(const Konversation::ServerGroupSettings*)));
     connect(m_viewContainer, SIGNAL(setWindowCaption(const QString&)), this, SLOT(setCaption(const QString&)));
-    connect(this, SIGNAL(serverStateChanged(Server*, Server::State)), m_viewContainer, SLOT(serverStateChanged(Server*, Server::State)));
-    connect(this, SIGNAL(insertRememberLine()), m_viewContainer, SLOT(insertRememberLine()));
-    connect(this, SIGNAL(insertRememberLine(Server*)), m_viewContainer, SLOT(insertRememberLine(Server*)));
-
+    connect(KonversationApplication::instance()->getConnectionManager(),
+            SIGNAL(connectionChangedState(Server*, Konversation::ConnectionState)),
+            m_viewContainer, SLOT(connectionStateChanged(Server*, Konversation::ConnectionState)));
+    connect(this, SIGNAL(triggerRememberLine()), m_viewContainer, SLOT(insertRememberLine()));
+    connect(this, SIGNAL(triggerRememberLines(Server*)), m_viewContainer, SLOT(insertRememberLines(Server*)));
+    connect(this, SIGNAL(cancelRememberLine()), m_viewContainer, SLOT(cancelRememberLine()));
+    connect(this, SIGNAL(insertMarkerLine()), m_viewContainer, SLOT(insertMarkerLine()));
 
     // Set up status bar
     m_statusBar = new KonversationStatusBar(this);
@@ -175,6 +185,9 @@ KonversationMainWindow::KonversationMainWindow() : KMainWindow(0,"main_window",
     action->setEnabled(false);
     action = new KAction(i18n("Close &Tab"),"tab_remove",KShortcut("Ctrl+w"), m_viewContainer, SLOT(closeCurrentView()),actionCollection(),"close_tab");
     action->setEnabled(false);
+    action = new KAction(i18n("Next Active Tab"), 0, KShortcut("Ctrl+Alt+Space"), m_viewContainer, SLOT(showNextActiveView()),
+                         actionCollection(), "next_active_tab");
+    action->setEnabled(false);
 
     if (Preferences::tabPlacement()==Preferences::Left)
     {
@@ -211,9 +224,15 @@ KonversationMainWindow::KonversationMainWindow() : KMainWindow(0,"main_window",
         action2->setToolTip("Move this tab");
     }
 
+    action = new KAction(i18n("Rejoin Channel"), 0, m_viewContainer, SLOT(rejoinChannel()), actionCollection(), "rejoin_channel");
+    action->setEnabled(false);
+
     action = new KToggleAction(i18n("Enable Notifications"), 0, 0, m_viewContainer, SLOT(toggleViewNotifications()), actionCollection(), "tab_notifications");
     action->setEnabled(false);
 
+    action = new KToggleAction(i18n("Join on Connect"), 0, 0, m_viewContainer, SLOT(toggleAutoJoin()), actionCollection(), "tab_autojoin");
+    action->setEnabled(false);
+
     KSelectAction* selectAction = new KSelectAction(i18n("Set Encoding"), "charset", 0, actionCollection(), "tab_encoding");
     selectAction->setEditable(false);
     QStringList encodingDescs = Konversation::IRCCharsets::self()->availableEncodingDescriptiveNames();
@@ -222,10 +241,6 @@ KonversationMainWindow::KonversationMainWindow() : KMainWindow(0,"main_window",
     selectAction->setEnabled(false);
     connect(selectAction, SIGNAL(activated(int)), m_viewContainer, SLOT(changeViewCharset(int)));
 
-    selectAction = new KSelectAction(i18n("Switch To"), 0, 0, actionCollection(), "switch_to_tab");
-    selectAction->setEditable(false);
-    connect(selectAction, SIGNAL(activated(int)), m_viewContainer, SLOT(goToView(int)));
-
     QSignalMapper* tabSelectionMapper = new QSignalMapper(this);
     connect(tabSelectionMapper, SIGNAL(mapped(int)), m_viewContainer, SLOT(goToView(int)));
 
@@ -236,6 +251,9 @@ KonversationMainWindow::KonversationMainWindow() : KMainWindow(0,"main_window",
         tabSelectionMapper->setMapping( tabSelectionAction, i-1);
     }
 
+    action = new KAction(i18n("Clear &Marker Lines"), 0, KShortcut("CTRL+SHIFT+R"), m_viewContainer, SLOT(clearViewLines()),actionCollection(),"clear_lines");
+    action->setToolTip(i18n("Clear marker lines in the current tab"));
+    action->setEnabled(false);
     action = new KAction(i18n("&Clear Window"), 0, KShortcut("Ctrl+L"), m_viewContainer, SLOT(clearView()),actionCollection(),"clear_window");
     action->setToolTip(i18n("Clear the contents of the current tab"));
     action->setEnabled(false);
@@ -243,8 +261,8 @@ KonversationMainWindow::KonversationMainWindow() : KMainWindow(0,"main_window",
     action->setToolTip(i18n("Clear the contents of all open tabs"));
     action->setEnabled(false);
 
-    KAction* awayAction = new KAction(i18n("Set &Away Globally")/*, "konversationaway"*/, KShortcut("Ctrl+Shift+A"),
-        static_cast<KonversationApplication *>(kapp), SLOT(toggleAway()), actionCollection(),"toggle_away");
+    KToggleAction* awayAction = new KToggleAction(i18n("Global Away"), KShortcut("Ctrl+Shift+A"), actionCollection(), "toggle_away");
+    connect(awayAction, SIGNAL(toggled(bool)), static_cast<KonversationApplication*>(kapp)->getAwayManager(), SLOT(toggleGlobalAway(bool)));
     awayAction->setEnabled(false);
 
     action = new KAction(i18n("&Join Channel..."), "add", KShortcut("Ctrl+J"), m_viewContainer, SLOT(showJoinChannelDialog()), actionCollection(), "join_channel");
@@ -255,11 +273,13 @@ KonversationMainWindow::KonversationMainWindow() : KMainWindow(0,"main_window",
     action->setEnabled(false);
     action = KStdAction::findNext(m_viewContainer, SLOT(findNextText()), actionCollection());
     action->setEnabled(false);
+    action = KStdAction::findPrev(m_viewContainer, SLOT(findPrevText()), actionCollection());
+    action->setEnabled(false);
 
     action = new KAction(i18n("&IRC Color..."), "colorize", CTRL+Key_K, m_viewContainer, SLOT(insertIRCColor()), actionCollection(), "irc_colors");
     action->setToolTip(i18n("Set the color of your current IRC message"));
     action->setEnabled(false);
-    action = new KAction(i18n("&Remember Line"), 0,  KShortcut("Ctrl+R") , m_viewContainer, SLOT(insertRememberLine()), actionCollection(), "insert_remember_line");
+    action = new KAction(i18n("&Marker Line"), 0,  KShortcut("Ctrl+R") , m_viewContainer, SLOT(insertMarkerLine()), actionCollection(), "insert_marker_line");
     action->setToolTip(i18n("Insert a horizontal line into the current tab that only you can see"));
     action->setEnabled(false);
     action = new KAction(i18n("Special &Character..."), "char", KShortcut("Alt+Shift+C"), m_viewContainer, SLOT(insertCharacter()), actionCollection(), "insert_character");
@@ -277,6 +297,7 @@ KonversationMainWindow::KonversationMainWindow() : KMainWindow(0,"main_window",
     // set up system tray
     m_trayIcon = new Konversation::TrayIcon(this);
     connect(this, SIGNAL(endNotification()), m_trayIcon, SLOT(endNotification()));
+    connect(KonversationApplication::instance(), SIGNAL(iconChanged(int)), m_trayIcon, SLOT(updateAppearance()));
     connect(m_trayIcon, SIGNAL(quitSelected()), this, SLOT(quitProgram()));
     KPopupMenu *trayMenu = m_trayIcon->contextMenu();
     #ifdef USE_KNOTIFY
@@ -299,7 +320,6 @@ KonversationMainWindow::KonversationMainWindow() : KMainWindow(0,"main_window",
 
     // Bookmarks
     m_bookmarkHandler = new KonviBookmarkHandler(this);
-    connect(m_bookmarkHandler,SIGNAL(openURL(const QString&,const QString&)),this,SLOT(openURL(const QString&,const QString&)));
 
     // set up KABC with a nice gui error dialog
     KABC::GuiErrorHandler *m_guiErrorHandler = new KABC::GuiErrorHandler(this);
@@ -314,22 +334,38 @@ KonversationMainWindow::KonversationMainWindow() : KMainWindow(0,"main_window",
 
 KonversationMainWindow::~KonversationMainWindow()
 {
-    Preferences::writeConfig();
+    delete m_viewContainer;
 }
 
 int KonversationMainWindow::confirmQuit()
 {
-    KonversationApplication *konvApp=static_cast<KonversationApplication *>(kapp);
+    KonversationApplication* konvApp = static_cast<KonversationApplication*>(kapp);
 
-    QPtrList<Server> serverList = konvApp->getServerList();
-    if (serverList.isEmpty()) return KMessageBox::Continue;
+    if (konvApp->getConnectionManager()->connectionCount() == 0)
+        return KMessageBox::Continue;
+
+    int result = KMessageBox::Cancel;
+
+    if (!KMessageBox::shouldBeShownContinue("systemtrayquitKonversation")
+         && konvApp->getDccTransferManager()->hasActiveTransfers())
+    {
+        result = KMessageBox::warningContinueCancel(
+            this,
+            i18n("<qt>You have active DCC file transfers. Are you sure you want to quit <b>Konversation</b>?</qt>"),
+            i18n("Confirm Quit"),
+            i18n("Quit"),
+            "QuitWithActiveDccTransfers");
+    }
+    else
+    {
+        result = KMessageBox::warningContinueCancel(
+            this,
+            i18n("<qt>Are you sure you want to quit <b>Konversation</b>?</qt>"),
+            i18n("Confirm Quit"),
+            i18n("Quit"),
+            "systemtrayquitKonversation");
+    }
 
-    int result=KMessageBox::warningContinueCancel(
-        this,
-        i18n("<qt>Are you sure you want to quit <b>Konversation</b>?</qt>"),
-        i18n("Confirm Quit"),
-        i18n("Quit"),
-        "systemtrayquitKonversation");
     return result;
 }
 
@@ -346,48 +382,45 @@ void KonversationMainWindow::quitProgram()
 
 bool KonversationMainWindow::queryClose()
 {
-    KonversationApplication* konv_app = static_cast<KonversationApplication*>(kapp);
+    KonversationApplication* konvApp = static_cast<KonversationApplication*>(kapp);
 
-    if (konv_app->sessionSaving() || sender() == m_trayIcon)
-        m_closeApp = true;
-
-    if (Preferences::showTrayIcon() && !m_closeApp)
+    if (!konvApp->sessionSaving())
     {
-        KMessageBox::information( this,
-            i18n("<p>Closing the main window will keep Konversation running in the system tray. "
-            "Use <b>Quit</b> from the <b>Konversation</b> menu to quit the application.</p>"),
-            i18n( "Docking in System Tray" ),  "HideOnCloseInfo" );
-        hide();
+        if (sender() == m_trayIcon)
+            m_closeApp = true;
 
-        return false;
+        if (Preferences::showTrayIcon() && !m_closeApp)
+        {
+            KMessageBox::information( this,
+                i18n("<p>Closing the main window will keep Konversation running in the system tray. "
+                "Use <b>Quit</b> from the <b>Konversation</b> menu to quit the application.</p>"),
+                i18n( "Docking in System Tray" ),  "HideOnCloseInfo" );
+            hide();
+
+            return false;
+        }
+
+        if (!Preferences::showTrayIcon() && confirmQuit() == KMessageBox::Cancel)
+            return false;
     }
 
-    if (!Preferences::showTrayIcon() && confirmQuit() == KMessageBox::Cancel)
-        return false;
-
-    m_viewContainer->silenceViews();
-
-    // send quit to all servers
-    emit quitServer();
-
     return true;
 }
 
 void KonversationMainWindow::hideEvent(QHideEvent *e)
 {
-    if (Preferences::autoInsertRememberLineAfterMinimizing())
-        emit insertRememberLine();
+    emit triggerRememberLine();
 
     m_statusBar->clearMainLabelTempText();
 
     KMainWindow::hideEvent(e);
 }
 
-void KonversationMainWindow::focusOutEvent(QFocusEvent* e)
+void KonversationMainWindow::showEvent(QShowEvent *e)
 {
-    m_statusBar->clearMainLabelTempText();
+    emit cancelRememberLine();
 
-    KMainWindow::focusOutEvent(e);
+    KMainWindow::showEvent(e);
 }
 
 void KonversationMainWindow::leaveEvent(QEvent* e)
@@ -399,7 +432,18 @@ void KonversationMainWindow::leaveEvent(QEvent* e)
 
 bool KonversationMainWindow::event(QEvent* e)
 {
-    if (e->type() == QEvent::WindowActivate) emit endNotification();
+    if (e->type() == QEvent::WindowActivate)
+    {
+        emit endNotification();
+        emit cancelRememberLine();
+    }
+    else if(e->type() == QEvent::WindowDeactivate)
+    {
+        m_statusBar->clearMainLabelTempText();
+
+        if (kapp->activeModalWidget() == 0)
+            emit triggerRememberLine();
+    }
 
     return KMainWindow::event(e);
 }
@@ -496,7 +540,7 @@ void KonversationMainWindow::openKeyBindings()
     actionCollection()->action("toggle_away")->setText(i18n("Toggle Away Globally"));
     actionCollection()->action("irc_colors")->setText(i18n("Insert &IRC Color..."));
     actionCollection()->action("insert_character")->setText(i18n("Insert Special &Character..."));
-    actionCollection()->action("insert_remember_line")->setText(i18n("Insert &Remember Line"));
+    actionCollection()->action("insert_marker_line")->setText(i18n("Insert &Marker Line"));
     QString openChannelListString = actionCollection()->action("open_channel_list")->text();
     actionCollection()->action("open_channel_list")->setText(i18n("&Channel List"));
     QString openLogFileString = actionCollection()->action("open_logfile")->text();
@@ -510,7 +554,7 @@ void KonversationMainWindow::openKeyBindings()
     actionCollection()->action("toggle_away")->setText(i18n("Set &Away Globally"));
     actionCollection()->action("irc_colors")->setText(i18n("&IRC Color..."));
     actionCollection()->action("insert_character")->setText(i18n("Special &Character..."));
-    actionCollection()->action("insert_remember_line")->setText(i18n("&Remember Line"));
+    actionCollection()->action("insert_marker_line")->setText(i18n("&Marker Line"));
     actionCollection()->action("open_channel_list")->setText(openChannelListString);
     actionCollection()->action("open_logfile")->setText(openLogFileString);
 }
@@ -520,13 +564,17 @@ void KonversationMainWindow::openServerList()
     if (!m_serverListDialog)
     {
         m_serverListDialog = new Konversation::ServerListDialog(this);
-        KonversationApplication *konvApp = static_cast<KonversationApplication *>(KApplication::kApplication());
-        connect(m_serverListDialog, SIGNAL(serverGroupsChanged()), konvApp, SLOT(saveOptions()));
-        connect(m_serverListDialog, SIGNAL(serverGroupsChanged()), this, SIGNAL(prefsChanged()));
-        connect(m_serverListDialog, SIGNAL(serverGroupsChanged()), m_viewContainer, SLOT(updateViews()));
-        connect(m_serverListDialog, SIGNAL(connectToServer(int)), konvApp, SLOT(connectToServer(int)));
-        connect(m_serverListDialog, SIGNAL(connectToServer(int, Konversation::ServerSettings)), konvApp, SLOT(connectToServer(int, Konversation::ServerSettings)));
-        connect(konvApp, SIGNAL(closeServerList()), m_serverListDialog, SLOT(slotClose()));
+        KonversationApplication* konvApp = static_cast<KonversationApplication*>(kapp);
+
+        connect(m_serverListDialog, SIGNAL(serverGroupsChanged(const Konversation::ServerGroupSettings*)),
+                konvApp, SIGNAL(serverGroupsChanged(const Konversation::ServerGroupSettings*)));
+        connect(konvApp, SIGNAL(serverGroupsChanged(const Konversation::ServerGroupSettings*)),
+                m_serverListDialog, SLOT(updateServerList()));
+        connect(m_serverListDialog, SIGNAL(connectTo(Konversation::ConnectionFlag, int)),
+                konvApp->getConnectionManager(), SLOT(connectTo(Konversation::ConnectionFlag, int)));
+        connect(m_serverListDialog, SIGNAL(connectTo(Konversation::ConnectionFlag, ConnectionSettings&)),
+                konvApp->getConnectionManager(), SLOT(connectTo(Konversation::ConnectionFlag, ConnectionSettings&)));
+        connect(konvApp->getConnectionManager(), SIGNAL(closeServerList()), m_serverListDialog, SLOT(slotClose()));
     }
 
     m_serverListDialog->show();
@@ -540,16 +588,20 @@ void KonversationMainWindow::openQuickConnectDialog()
 // open the preferences dialog and show the watched nicknames page
 void KonversationMainWindow::openNotify()
 {
-  openPrefsDialog();
-  if (m_settingsDialog) m_settingsDialog->openWatchedNicknamesPage();
+    openPrefsDialog();
+    if (m_settingsDialog) m_settingsDialog->openWatchedNicknamesPage();
 }
 
 void KonversationMainWindow::openIdentitiesDialog()
 {
     Konversation::IdentityDialog dlg(this);
 
-    if((dlg.exec() == KDialog::Accepted) && m_serverListDialog)
-        m_serverListDialog->updateServerList();
+    if (dlg.exec() == KDialog::Accepted)
+    {
+        if (m_serverListDialog)
+            m_serverListDialog->updateServerList();
+        m_viewContainer->updateViewEncoding(m_viewContainer->getFrontView());
+    }
 }
 
 IdentityPtr KonversationMainWindow::editIdentity(IdentityPtr identity)
@@ -575,14 +627,13 @@ void KonversationMainWindow::openNotifications()
     #endif
 }
 
-void KonversationMainWindow::notifyAction(const QString& serverName,const QString& nick)
+void KonversationMainWindow::notifyAction(const QString& serverName, const QString& nick)
 {
-    KonversationApplication* konv_app=static_cast<KonversationApplication*>(KApplication::kApplication());
-    Server* server=konv_app->getServerByName(serverName);
-    server->notifyAction(nick);
+    KonversationApplication* konvApp = static_cast<KonversationApplication*>(kapp);
+    Server* server = konvApp->getConnectionManager()->getServerByName(serverName);
+    if (server) server->notifyAction(nick);
 }
 
-
 // TODO: Let an own class handle notify things
 void KonversationMainWindow::setOnlineList(Server* notifyServer,const QStringList& /*list*/, bool /*changed*/)
 {
@@ -590,37 +641,6 @@ void KonversationMainWindow::setOnlineList(Server* notifyServer,const QStringLis
     // FIXME  if (changed && nicksOnlinePanel) newText(nicksOnlinePanel, QString::null, true);
 }
 
-void KonversationMainWindow::openURL(const QString& url, const QString& /*title*/)
-{
-    QString urlN = url;
-    urlN.remove("irc://");
-
-    QString host = urlN.section('/',0,0);
-    KonviIpHelper hostParser(host);
-    host = hostParser.host();
-
-    QString port = (hostParser.port().isEmpty() ? QString("6667") : hostParser.port());
-
-    QString channel = urlN.section('/',1,1);
-    QString password;
-
-    if (Preferences::isServerGroup(host))
-    {
-        Server* newServer = KonversationApplication::instance()->connectToServerGroup(host);
-
-        if (!newServer->isConnected())
-        {
-            newServer->setAutoJoin(true);
-            newServer->setAutoJoinChannel(channel);
-            newServer->setAutoJoinChannelKey(password);
-        }
-        else if (!channel.isEmpty())
-            newServer->queue("JOIN " + channel + ' ' + password);
-    }
-    else
-        KonversationApplication::instance()->quickConnectToServer(host,port,channel,"",password);
-}
-
 QString KonversationMainWindow::currentURL(bool passNetwork)
 {
     return m_viewContainer->currentViewURL(passNetwork);
