@@ -6,19 +6,20 @@
 */
 
 /*
-  copyright: (C) 2005 by Dario Abatianni
-  email:     eisfuchs@tigress.com
+  Copyright (C) 2002 Dario Abatianni <eisfuchs@tigress.com>
+  Copyright (C) 2007-2007 Eike Hein <hein@kde.org>
 */
 
 #ifndef KONV_IFACE_H
 #define KONV_IFACE_H
 
+#include "ignore.h"
+
 #include <qobject.h>
 #include <qstringlist.h>
 
 #include <dcopobject.h>
 
-#include "ignore.h"
 
 class KonvIface : virtual public DCOPObject
 {
@@ -35,13 +36,16 @@ class KonvIface : virtual public DCOPObject
     virtual void info(const QString& string) = 0;
     virtual void debug(const QString& string) = 0;
     virtual void error(const QString& string) = 0;
-    virtual void insertRememberLine() = 0;
-    virtual void connectToServer(const QString& url, int port, const QString& channel, const QString& password) = 0;
+    virtual void insertMarkerLine() = 0;
+    virtual void connectToServer(const QString& address, int port, const QString& channel, const QString& password) = 0;
     virtual QString getNickname (const QString &serverid) = 0;
     virtual QString getAnyNickname () = 0;
     virtual QStringList listServers() = 0;
     virtual QStringList listConnectedServers() = 0;
     virtual QString getChannelEncoding(const QString& server, const QString& channel) = 0;
+
+    virtual void setScreenSaverStarted() = 0;
+    virtual void setScreenSaverStopped() = 0;
 };
 
 class KonvIdentityIface : virtual public DCOPObject
@@ -65,6 +69,8 @@ class KonvIdentityIface : virtual public DCOPObject
     virtual void setNicknameList(const QString &identity, const QStringList& newList) = 0;
     virtual QStringList getNicknameList(const QString &identity) = 0;
 
+    virtual void setQuitReason(const QString &identity, const QString& reason) = 0;
+    virtual QString getQuitReason(const QString &identity) = 0;
     virtual void setPartReason(const QString &identity, const QString& reason) = 0;
     virtual QString getPartReason(const QString &identity) = 0;
     virtual void setKickReason(const QString &identity, const QString& reason) = 0;
