@@ -13,37 +13,34 @@
 #ifndef NICKLISTBEHAVIOR_Config_H
 #define NICKLISTBEHAVIOR_Config_H
 
-#include <qobject.h>
 #include "konvisettingspage.h"
 #include "nicklistbehavior_preferencesui.h"
 
-/**
-  @author Dario Abatianni <eisfuchs@tigress.com>
- */
+#include <qobject.h>
+
 
 class NicklistBehavior_Config : public NicklistBehavior_ConfigUI, public KonviSettingsPage
 {
-  Q_OBJECT
+    Q_OBJECT
 
-  public:
-    NicklistBehavior_Config(QWidget *parent = 0, const char *name = 0);
-    ~NicklistBehavior_Config();
+    public:
+        explicit NicklistBehavior_Config(QWidget *parent = 0, const char *name = 0);
+        ~NicklistBehavior_Config();
 
-    virtual void saveSettings();
-    virtual void loadSettings();
-    virtual void restorePageToDefaults();
+        virtual void saveSettings();
+        virtual void loadSettings();
+        virtual void restorePageToDefaults();
 
-    virtual bool hasChanged();
+        virtual bool hasChanged();
 
-  private:
-    void setNickList(const QString &sortingOrder);
-    QString currentSortingOrder();
+    private:
+        void setNickList(const QString &sortingOrder);
+        QString currentSortingOrder();
 
-    QString m_oldSortingOrder;
-
-  signals:
-    void modified();
+        QString m_oldSortingOrder;
 
+    signals:
+        void modified();
 };
 
 #endif
