@@ -14,7 +14,20 @@
   email:     eisfuchs@tigress.com
 */
 
-// Qt includes.
+#include "nicksonline.h"
+#include "channel.h"
+#include "server.h"
+#include "konversationapplication.h"
+#include "connectionmanager.h"
+#include "images.h"
+#include "query.h"
+#include "linkaddressbook/linkaddressbookui.h"
+#include "linkaddressbook/addressbook.h"
+#include "linkaddressbook/nicksonlinetooltip.h"
+#include "konversationmainwindow.h"
+#include "viewcontainer.h"
+#include "nicksonlineitem.h"
+
 #include <qlayout.h>
 #include <qstringlist.h>
 #include <qhbox.h>
@@ -24,7 +37,7 @@
 #include <qptrlist.h>
 #include <qwhatsthis.h>
 
-// KDE includes.
+
 #include <kdebug.h>
 #include <klocale.h>
 #include <kdialog.h>
@@ -33,30 +46,14 @@
 #include <kprocess.h>
 #include <kmessagebox.h>
 
-// Konversation includes.
-#include "channel.h"
-#include "nicksonline.h"
-#include "server.h"
-#include "konversationapplication.h"
-#include "images.h"
-#include "linkaddressbook/linkaddressbookui.h"
-#include "linkaddressbook/addressbook.h"
-#include "linkaddressbook/nicksonlinetooltip.h"
-#include "konversationmainwindow.h"
 
 NicksOnline::NicksOnline(QWidget* parent): ChatWindow(parent)
 {
     setName(i18n("Watched Nicks Online"));
     setType(ChatWindow::NicksOnline);
 
-    // Convenience constant for the internationalized string "Offline".
-    c_i18nOffline = i18n("Offline");
-
     m_nickListView=new KListView(this);
 
-    // TODO: Need to derive from KListView and override sort() method in order to sort in
-    // locale-aware order.
-
     // Set to false every 8 seconds to permit a whois on watched nicks lacking information.
     // Remove when server or addressbook does this automatically.
     m_whoisRequested = true;
@@ -64,9 +61,12 @@ NicksOnline::NicksOnline(QWidget* parent): ChatWindow(parent)
     m_nickListView->addColumn(i18n("Network/Nickname/Channel"));
     m_kabcIconSet = KGlobal::iconLoader()->loadIconSet("kaddressbook",KIcon::Small);
     m_nickListView->addColumn(i18n("Additional Information"));
-    m_nickListView->setFullWidth(false);
+    m_nickListView->setFullWidth(true);
     m_nickListView->setRootIsDecorated(true);
     m_nickListView->setShowToolTips(false);
+    m_nickListView->setShadeSortColumn(true);
+    m_nickListView->setShowSortIndicator(true);
+
     QString nickListViewWT = i18n(
         "<p>These are all the nicknames on your Nickname Watch list, listed under the "
         "server network they are connected to.  The list also includes the nicknames "
@@ -165,17 +165,49 @@ KListView* NicksOnline::getNickListView()
 }
 
 /**
- * Returns the named child of parent item in a KListView.
- * @param parent            Pointer to a QListViewItem.
- * @param name              The name in the desired child QListViewItem.  Name
- *                          is assumed to be in column 0 of the item.
+ * Returns the named child of parent item in a NicksOnlineItem
+ * @param parent            Pointer to a NicksOnlineItem.
+ * @param name              The name in the desired child QListViewItem, must be in column 0.
+ * @param type              The type of entry to be found
  * @return                  Pointer to the child QListViewItem or 0 if not found.
  */
-QListViewItem* NicksOnline::findItemChild(const QListViewItem* parent, const QString& name)
+QListViewItem* NicksOnline::findItemChild(const QListViewItem* parent, const QString& name, NicksOnlineItem::NickListViewColumn type)
 {
     if (!parent) return 0;
     QListViewItem* child;
     for (child = parent->firstChild(); (child) ; child = child->nextSibling())
+    {
+        if(static_cast<NicksOnlineItem*>(child)->type() == type && child->text(0) == name) return child;
+    }
+    return 0;
+}
+
+/**
+ * Returns the first occurrence of a child item of a given type in a NicksOnlineItem
+ * @param parent            Pointer to a NicksOnlineItem.
+ * @param type              The type of entry to be found
+ * @return                  Pointer to the child QListViewItem or 0 if not found.
+ */
+QListViewItem* NicksOnline::findItemType(const QListViewItem* parent, NicksOnlineItem::NickListViewColumn type)
+{
+    if (!parent) return 0;
+    QListViewItem* child;
+    for (child = parent->firstChild(); (child) ; child = child->nextSibling())
+    {
+        if(static_cast<NicksOnlineItem*>(child)->type() == type) return child;
+    }
+    return 0;
+}
+
+/**
+ * Returns a pointer to the network QListViewItem with the given name.
+ * @param name              The name of the network, assumed to be in column 0 of the item.
+ * @return                  Pointer to the QListViewItem or 0 if not found.
+ */
+QListViewItem* NicksOnline::findNetworkRoot(const QString& name)
+{
+    QListViewItem* child;
+    for (child = getNickListView()->firstChild(); (child) ; child = child->nextSibling())
     {
         if (child->text(0) == name) return child;
     }
@@ -209,23 +241,23 @@ bool& needWhois)
         {
             niInfo += i18n("Away");
             if (!nickInfo->getAwayMessage().isEmpty())
-                niInfo += "(" + nickInfo->getAwayMessage() + ")";
+                niInfo += '(' + nickInfo->getAwayMessage() + ')';
         }
         if (!nickInfo->getHostmask().isEmpty())
-            niInfo += " " + nickInfo->getHostmask();
+            niInfo += ' ' + nickInfo->getHostmask();
         if (!nickInfo->getRealName().isEmpty())
-            niInfo += " (" + nickInfo->getRealName() + ")";
+            niInfo += " (" + nickInfo->getRealName() + ')';
         if (!nickInfo->getNetServer().isEmpty())
         {
             niInfo += i18n( " online via %1" ).arg( nickInfo->getNetServer() );
             if (!nickInfo->getNetServerInfo().isEmpty())
-                niInfo += " (" + nickInfo->getNetServerInfo() + ")";
+                niInfo += " (" + nickInfo->getNetServerInfo() + ')';
         }
         if (!nickInfo->getOnlineSince().isNull())
             niInfo += i18n( " since %1" ).arg( nickInfo->getPrettyOnlineSince() );
     }
     needWhois = niInfo.isEmpty();
-    if (!info.isEmpty() && !needWhois) info += " ";
+    if (!info.isEmpty() && !needWhois) info += ' ';
     return info + niInfo;
 }
 
@@ -237,14 +269,12 @@ void NicksOnline::updateServerOnlineList(Server* servr)
 {
     bool newNetworkRoot = false;
     QString serverName = servr->getServerName();
-    // TODO: The method name "getServerGroup" is a misnomer.  Actually returns the
-    // network for a server.
-    QString networkName = servr->getServerGroup();
-    QListViewItem* networkRoot = m_nickListView->findItem(networkName, nlvcNetwork);
+    QString networkName = servr->getDisplayName();
+    QListViewItem* networkRoot = findNetworkRoot(networkName);
     // If network is not in our list, add it.
     if (!networkRoot)
     {
-        networkRoot = new KListViewItem(m_nickListView,networkName);
+        networkRoot = new NicksOnlineItem(NicksOnlineItem::NetworkRootItem,m_nickListView,networkName);
         newNetworkRoot = true;
     }
     // Store server name in hidden column.
@@ -257,9 +287,12 @@ void NicksOnline::updateServerOnlineList(Server* servr)
     if (!serverList.contains(serverName)) serverList.append(serverName);
     networkRoot->setText(nlvcAdditionalInfo, serverList.join(","));
     // Get item in nicklistview for the Offline branch.
-    QListViewItem* offlineRoot = findItemChild(networkRoot, c_i18nOffline);
-    if (!offlineRoot) offlineRoot = new KListViewItem(networkRoot, c_i18nOffline);
-    offlineRoot->setText(nlvcServerName, serverName);
+    QListViewItem* offlineRoot = findItemType(networkRoot, NicksOnlineItem::OfflineItem);
+    if (!offlineRoot)
+    {
+        offlineRoot = new NicksOnlineItem(NicksOnlineItem::OfflineItem,networkRoot,i18n("Offline"));
+        offlineRoot->setText(nlvcServerName, serverName);
+    }
 
     // Get watch list.
     QStringList watchList = servr->getWatchList();
@@ -282,11 +315,11 @@ void NicksOnline::updateServerOnlineList(Server* servr)
             bool needWhois = false;
             QString nickAdditionalInfo = getNickAdditionalInfo(nickInfo, addressee, needWhois);
             // Remove from offline branch if present.
-            QListViewItem* item = findItemChild(offlineRoot, nickname);
+            QListViewItem* item = findItemChild(offlineRoot, nickname, NicksOnlineItem::NicknameItem);
             if (item) delete item;
             // Add to network if not already added.
-            QListViewItem* nickRoot = findItemChild(networkRoot, nickname);
-            if (!nickRoot) nickRoot = new KListViewItem(networkRoot, nickname, nickAdditionalInfo);
+            QListViewItem* nickRoot = findItemChild(networkRoot, nickname, NicksOnlineItem::NicknameItem);
+            if (!nickRoot) nickRoot = new NicksOnlineItem(NicksOnlineItem::NicknameItem,networkRoot, nickname, nickAdditionalInfo);
             nickRoot->setText(nlvcAdditionalInfo, nickAdditionalInfo);
             nickRoot->setText(nlvcServerName, serverName);
             // If no additional info available, request a WHOIS on the nick.
@@ -325,8 +358,8 @@ void NicksOnline::updateServerOnlineList(Server* servr)
                 if (channelNick->isOp()) nickMode = nickMode + i18n(" Operator");
                 if (channelNick->isOwner()) nickMode = nickMode + i18n(" Owner");
                 if (channelNick->isAdmin()) nickMode = nickMode + i18n(" Admin");
-                QListViewItem* channelItem = findItemChild(nickRoot, channelName);
-                if (!channelItem) channelItem = new KListViewItem(nickRoot,
+                QListViewItem* channelItem = findItemChild(nickRoot, channelName, NicksOnlineItem::ChannelItem);
+                if (!channelItem) channelItem = new NicksOnlineItem(NicksOnlineItem::ChannelItem,nickRoot,
                         channelName, nickMode);
                 channelItem->setText(nlvcAdditionalInfo, nickMode);
 
@@ -358,11 +391,11 @@ void NicksOnline::updateServerOnlineList(Server* servr)
         {
             // Nick is offline.
             // Remove from online nicks, if present.
-            QListViewItem* item = findItemChild(networkRoot, nickname);
+            QListViewItem* item = findItemChild(networkRoot, nickname, NicksOnlineItem::NicknameItem);
             if (item) delete item;
             // Add to offline list if not already listed.
-            QListViewItem* nickRoot = findItemChild(offlineRoot, nickname);
-            if (!nickRoot) nickRoot = new KListViewItem(offlineRoot, nickname);
+            QListViewItem* nickRoot = findItemChild(offlineRoot, nickname, NicksOnlineItem::NicknameItem);
+            if (!nickRoot) nickRoot = new NicksOnlineItem(NicksOnlineItem::NicknameItem,offlineRoot, nickname);
             nickRoot->setText(nlvcServerName, serverName);
             // Get addressbook entry for the nick.
             KABC::Addressee addressee = servr->getOfflineNickAddressee(nickname);
@@ -384,22 +417,29 @@ void NicksOnline::updateServerOnlineList(Server* servr)
     while (item)
     {
         QListViewItem* nextItem = item->nextSibling();
-        QString nickname = item->text(nlvcNick);
-        if (nickname != c_i18nOffline)
+        if (static_cast<NicksOnlineItem*>(item)->type() != NicksOnlineItem::OfflineItem)
         {
+            QString nickname = item->text(nlvcNick);
             if ((watchList.find(nickname) == watchList.end()) &&
                 (serverName == item->text(nlvcServerName))) delete item;
         }
         item = nextItem;
     }
     item = offlineRoot->firstChild();
-    while (item)
+
+    if(item) {
+        while (item)
+        {
+            QListViewItem* nextItem = item->nextSibling();
+            QString nickname = item->text(nlvcNick);
+            if ((watchList.find(nickname) == watchList.end()) &&
+                (serverName == item->text(nlvcServerName))) delete item;
+            item = nextItem;
+        }
+    }
+    else
     {
-        QListViewItem* nextItem = item->nextSibling();
-        QString nickname = item->text(nlvcNick);
-        if ((watchList.find(nickname) == watchList.end()) &&
-            (serverName == item->text(nlvcServerName))) delete item;
-        item = nextItem;
+        delete offlineRoot;
     }
     // Expand server if newly added to list.
     if (newNetworkRoot)
@@ -421,11 +461,11 @@ void NicksOnline::updateServerOnlineList(Server* servr)
 NickInfoPtr NicksOnline::getOnlineNickInfo(QString& networkName, QString& nickname)
 {
     // Get list of pointers to all servers.
-    KonversationApplication *konvApp=static_cast<KonversationApplication *>(KApplication::kApplication());
-    QPtrList<Server> serverList = konvApp->getServerList();
+    KonversationApplication* konvApp = static_cast<KonversationApplication*>(kapp);
+    QPtrList<Server> serverList = konvApp->getConnectionManager()->getServerList();
     for (Server* server = serverList.first(); server; server = serverList.next())
     {
-        if (server->getServerGroup() == networkName)
+        if (server->getDisplayName() == networkName)
         {
             NickInfoPtr nickInfo = server->getNickInfo(nickname);
             if (nickInfo) return nickInfo;
@@ -442,11 +482,11 @@ NickInfoPtr NicksOnline::getOnlineNickInfo(QString& networkName, QString& nickna
  */
 void NicksOnline::requestWhois(QString& networkName, QString& nickname)
 {
-    KonversationApplication *konvApp=static_cast<KonversationApplication *>(KApplication::kApplication());
-    QPtrList<Server> serverList = konvApp->getServerList();
+    KonversationApplication* konvApp = static_cast<KonversationApplication*>(kapp);
+    QPtrList<Server> serverList = konvApp->getConnectionManager()->getServerList();
     for (Server* server = serverList.first(); server; server = serverList.next())
     {
-        if (server->getServerGroup() == networkName)
+        if (server->getDisplayName() == networkName)
         {
             server->requestWhois(nickname);
             return;
@@ -459,9 +499,8 @@ void NicksOnline::requestWhois(QString& networkName, QString& nickname)
  */
 void NicksOnline::refreshAllServerOnlineLists()
 {
-    KonversationApplication *konvApp=static_cast<KonversationApplication *>(KApplication::kApplication());
-    // Get list of pointers to all servers.
-    QPtrList<Server> serverList = konvApp->getServerList();
+    KonversationApplication* konvApp = static_cast<KonversationApplication*>(kapp);
+    QPtrList<Server> serverList = konvApp->getConnectionManager()->getServerList();
     Server* server;
     // Remove servers no longer connected.
     QListViewItem* child = m_nickListView->firstChild();
@@ -480,7 +519,7 @@ void NicksOnline::refreshAllServerOnlineLists()
             for (server = serverList.first(); server; server = serverList.next())
             {
                 if ((server->getServerName() == serverName) &&
-                    (server->getServerGroup() == networkName)) found = true;
+                    (server->getDisplayName() == networkName)) found = true;
             }
             if (!found)
                 it = serverNameList.remove(it);
@@ -500,9 +539,6 @@ void NicksOnline::refreshAllServerOnlineLists()
     {
         updateServerOnlineList(server);
     }
-    // Adjust column widths.
-    m_nickListView->adjustColumn(nlvcNetworkNickChannel);
-    m_nickListView->adjustColumn(nlvcAdditionalInfo);
     // Refresh addressbook buttons.
     slotNickListView_SelectionChanged();
 }
@@ -537,17 +573,21 @@ void NicksOnline::processDoubleClick(QListViewItem* item)
 bool NicksOnline::getItemServerAndNick(const QListViewItem* item, QString& serverName, QString& nickname)
 {
     if (!item) return false;
+    // convert into NicksOnlineItem
+    const NicksOnlineItem* nlItem=static_cast<const NicksOnlineItem*>(item);
     // If on a network, return false;
-    if (!item->parent()) return false;
+    if (nlItem->type() == NicksOnlineItem::NetworkRootItem) return false;
+    // get server name
     serverName = item->text(nlvcServerName);
     // If on a channel, move up to the nickname.
-    if (serverName.isEmpty())
+    if (nlItem->type() == NicksOnlineItem::ChannelItem)
     {
         item = item->parent();
         serverName = item->text(nlvcServerName);
     }
     nickname = item->text(nlvcNick);
-    if (nickname == c_i18nOffline) return false;
+    // offline columns are not nick names
+    if (nlItem->type() == NicksOnlineItem::OfflineItem) return false;
     return true;
 }
 
@@ -555,10 +595,17 @@ NickInfoPtr NicksOnline::getNickInfo(const QListViewItem* item)
 {
     QString serverName;
     QString nickname;
+
     getItemServerAndNick(item, serverName, nickname);
-    if(!serverName || !nickname) return 0;
-    Server *server = static_cast<KonversationApplication *>(kapp)->getServerByName(serverName);
-    return server->getNickInfo(nickname);
+
+    if (!serverName || !nickname)
+        return 0;
+
+    Server* server = KonversationApplication::instance()->getConnectionManager()->getServerByName(serverName);
+
+    if (server) return server->getNickInfo(nickname);
+
+    return 0;
 }
 
 /**
@@ -573,12 +620,12 @@ NickInfoPtr NicksOnline::getNickInfo(const QListViewItem* item)
 QListViewItem* NicksOnline::getServerAndNickItem(const QString& serverName,
 const QString& nickname)
 {
-    KonversationApplication *konvApp=static_cast<KonversationApplication *>(KApplication::kApplication());
-    Server* server = konvApp->getServerByName(serverName);
-    QString networkName = server->getServerGroup();
+    Server* server = KonversationApplication::instance()->getConnectionManager()->getServerByName(serverName);
+    if (!server) return 0;
+    QString networkName = server->getDisplayName();
     QListViewItem* networkRoot = m_nickListView->findItem(networkName, nlvcNetwork);
     if (!networkRoot) return 0;
-    QListViewItem* nickRoot = findItemChild(networkRoot, nickname);
+    QListViewItem* nickRoot = findItemChild(networkRoot, nickname, NicksOnlineItem::NicknameItem);
     return nickRoot;
 }
 
@@ -610,13 +657,9 @@ void NicksOnline::doCommand(int id)
     }
 
     // Get the server object corresponding to the server name.
-    KonversationApplication *konvApp = static_cast<KonversationApplication *>(kapp);
-    Server* server = konvApp->getServerByName(serverName);
+    Server* server = KonversationApplication::instance()->getConnectionManager()->getServerByName(serverName);
 
-    if(!server)
-    {
-        return;
-    }
+    if (!server) return;
 
     // Get NickInfo object corresponding to the nickname.
     NickInfoPtr nickInfo = server->getNickInfo(nickname);
@@ -647,7 +690,7 @@ void NicksOnline::doCommand(int id)
             }
             else
             {
-                LinkAddressbookUI *linkaddressbookui = new LinkAddressbookUI(server->getMainWindow(), NULL, nickname, server->getServerName(), server->getServerGroup(), addressee.realName());
+                LinkAddressbookUI *linkaddressbookui = new LinkAddressbookUI(server->getViewContainer()->getWindow(), NULL, nickname, server->getServerName(), server->getDisplayName(), addressee.realName());
                 linkaddressbookui->show();
             }
             break;
@@ -665,13 +708,13 @@ void NicksOnline::doCommand(int id)
                         return;
                     }
 
-                    addressbook->unassociateNick(addressee, nickname, server->getServerName(), server->getServerGroup());
+                    addressbook->unassociateNick(addressee, nickname, server->getServerName(), server->getDisplayName());
                 }
                 else
                 {
                     addressee.setGivenName(nickname);
                     addressee.setNickName(nickname);
-                    addressbook->associateNickAndUnassociateFromEveryoneElse(addressee, nickname, server->getServerName(), server->getServerGroup());
+                    addressbook->associateNickAndUnassociateFromEveryoneElse(addressee, nickname, server->getServerName(), server->getDisplayName());
                 }
                 if(addressbook->saveTicket())
                 {
@@ -686,9 +729,8 @@ void NicksOnline::doCommand(int id)
         }
         case ciJoinChannel:
         {
-            // Channels have no nlvcServerName entry.
-            // We test if it is empty to see if we really have a channel name.
-            if (m_nickListView->selectedItem()->text(nlvcServerName).isEmpty())
+            // only join real channels
+            if (static_cast<NicksOnlineItem*>(m_nickListView->selectedItem())->type() == NicksOnlineItem::ChannelItem)
             {
                 QString contactChannel = m_nickListView->selectedItem()->text(nlvcChannel);
                 server->queue( "JOIN "+contactChannel );
@@ -698,6 +740,11 @@ void NicksOnline::doCommand(int id)
         case ciWhois:
             server->queue("WHOIS "+nickname);
             return;
+        case ciOpenQuery:
+            NickInfoPtr nickInfo = server->obtainNickInfo(nickname);
+            class Query* query = server->addQuery(nickInfo, true /*we initiated*/);
+            emit showView(query);
+            return;
     }
 
     refreshItem(item);
@@ -716,8 +763,7 @@ int NicksOnline::getNickAddressbookState(QListViewItem* item)
     QString nickname;
     if (getItemServerAndNick(item, serverName, nickname))
     {
-        Server *server =
-            static_cast<KonversationApplication *>(kapp)->getServerByName(serverName);
+        Server *server = KonversationApplication::instance()->getConnectionManager()->getServerByName(serverName);
         if (!server) return nsNotANick;
         NickInfoPtr nickInfo = server->getNickInfo(nickname);
         if (nickInfo)
@@ -827,6 +873,7 @@ void NicksOnline::slotNickListView_RightButtonClicked(QListViewItem* item, const
             m_popupMenu->insertItem(i18n("Create New C&ontact..."), ciAddressbookNew);
             m_popupMenu->insertSeparator();
             m_popupMenu->insertItem(i18n("&Whois"), ciWhois);
+            m_popupMenu->insertItem(i18n("Open &Query"), ciOpenQuery);
             if (item->text(nlvcServerName).isEmpty())
                 m_popupMenu->insertItem(i18n("&Join Channel"), ciJoinChannel);
             break;
@@ -841,6 +888,7 @@ void NicksOnline::slotNickListView_RightButtonClicked(QListViewItem* item, const
             m_popupMenu->insertItem(SmallIconSet("editdelete"), i18n("&Delete Association"), ciAddressbookDelete);
             m_popupMenu->insertSeparator();
             m_popupMenu->insertItem(i18n("&Whois"), ciWhois);
+            m_popupMenu->insertItem(i18n("Open &Query"), ciOpenQuery);
             if (item->text(nlvcServerName).isEmpty())
                 m_popupMenu->insertItem(i18n("&Join Channel"), ciJoinChannel);
             break;
@@ -883,8 +931,7 @@ void NicksOnline::refreshItem(QListViewItem* item)
     QString nickname;
     if (getItemServerAndNick(item, serverName, nickname))
     {
-        Server *server =
-            static_cast<KonversationApplication *>(kapp)->getServerByName(serverName);
+        Server *server = KonversationApplication::instance()->getConnectionManager()->getServerByName(serverName);
         if (server)
         {
             NickInfoPtr nickInfo = server->getNickInfo(nickname);
