@@ -16,16 +16,13 @@
 
 #include <kcombobox.h>
 
-/*
-  @author Dario Abatianni
-*/
 
 class TopicComboBox : public KComboBox
 {
     Q_OBJECT
 
-        public:
-        TopicComboBox(QWidget* parent);
+    public:
+        explicit TopicComboBox(QWidget* parent);
         ~TopicComboBox();
         void insertStringList(const QStringList& list);
 
