@@ -20,6 +20,7 @@
 
 #include <ksystemtray.h>
 
+
 class QTimer;
 class Server;
 
@@ -29,8 +30,9 @@ namespace Konversation
     class TrayIcon : public KSystemTray
     {
         Q_OBJECT
-            public:
-            TrayIcon(QWidget* parent = 0);
+
+        public:
+            explicit TrayIcon(QWidget* parent = 0);
             ~TrayIcon();
 
             bool notificationEnabled() { return m_notificationEnabled; }
@@ -39,13 +41,11 @@ namespace Konversation
             void startNotification();
             void endNotification();
             void setNotificationEnabled(bool notify) { m_notificationEnabled = notify; }
+            void updateAppearance();
 
         protected slots:
             void blinkTimeout();
 
-        protected:
-            void mousePressEvent(QMouseEvent *e);
-
         private:
             QTimer* m_blinkTimer;
             bool m_blinkOn;
