@@ -12,11 +12,18 @@
   email:     eisfuchs@tigress.com
 */
 
+#include "urlcatcher.h"
+#include "channel.h"
+#include "server.h"
+#include "konversationapplication.h"
+#include "viewcontainer.h"
+
 #include <qhbox.h>
 #include <qpushbutton.h>
 #include <qregexp.h>
 #include <qclipboard.h>
 #include <qwhatsthis.h>
+#include <qlayout.h>
 
 #include <kapplication.h>
 #include <kactionclasses.h>
@@ -28,15 +35,12 @@
 #include <kprocess.h>
 #include <kdeversion.h>
 #include <kshell.h>
+#include <klistviewsearchline.h>
 
-#include "urlcatcher.h"
-#include "channel.h"
-#include "server.h"
-#include "konversationapplication.h"
-#include "viewcontainer.h"
 
 UrlCatcher::UrlCatcher(QWidget* parent) : ChatWindow(parent)
 {
+    layout()->setAutoAdd(false);
     setName(i18n("URL Catcher"));
     setType(ChatWindow::UrlCatcher);
 
@@ -50,6 +54,9 @@ UrlCatcher::UrlCatcher(QWidget* parent) : ChatWindow(parent)
         "during this session.");
     QWhatsThis::add(urlListView, urlListViewWT);
 
+    searchWidget = new KListViewSearchLineWidget(urlListView, this, "search_line");
+    searchWidget->setEnabled(false);
+
     QHBox* buttonBox=new QHBox(this);
     buttonBox->setSpacing(spacing());
 
@@ -89,6 +96,10 @@ UrlCatcher::UrlCatcher(QWidget* parent) : ChatWindow(parent)
     saveListButton->setEnabled(false);
     clearListButton->setEnabled(false);
 
+    layout()->add(searchWidget);
+    layout()->add(urlListView);
+    layout()->add(buttonBox);
+
     urlSelected();
 }
 
@@ -119,6 +130,7 @@ void UrlCatcher::addUrl(const QString& who,const QString& url)
     new KListViewItem(urlListView,who,url);
     clearListButton->setEnabled(true);
     saveListButton->setEnabled(true);
+    searchWidget->setEnabled(true);
 }
 
 void UrlCatcher::openUrl(QListViewItem* item)
@@ -175,6 +187,7 @@ void UrlCatcher::deleteUrlClicked()
         {
             saveListButton->setEnabled(false);
             clearListButton->setEnabled(false);
+            searchWidget->setEnabled(false);
         }
     }
 }
@@ -192,8 +205,8 @@ void UrlCatcher::saveListClicked()
 {
     // Ask user for file name
     QString fileName=KFileDialog::getSaveFileName(
-        QString::null,
-        QString::null,
+        QString(),
+        QString(),
         this,
         i18n("Save URL List"));
 
