@@ -17,18 +17,15 @@
 
 #include <klistview.h>
 
-/*
-  @author Dario Abatianni
-*/
 
 class ValueListViewItem : public KListViewItem
 {
     public:
-        ValueListViewItem(int newValue,KListView* parent,QString label);
-        ValueListViewItem(int newValue,KListView* parent,QListViewItem* after,QString label);
+        ValueListViewItem(int newValue, KListView* parent, const QString& label);
+        ValueListViewItem(int newValue, KListView* parent, QListViewItem* after, const QString& label);
         ~ValueListViewItem();
 
-        const int getValue() const;
+        int getValue() const;
 
     protected:
         int m_value;
