@@ -6,22 +6,9 @@
 */
 
 /*
-  copyright: (C) 2006 by Eike Hein
-  email:     sho@eikehein.com
+  Copyright (C) 2006 Eike Hein <hein@kde.org>
 */
 
-#include <qheader.h>
-#include <qdragobject.h>
-#include <qlistview.h>
-#include <qptrlist.h>
-#include <qpoint.h>
-#include <qpainter.h>
-
-#include <kdebug.h>
-#include <klocale.h>
-#include <kglobalsettings.h>
-#include <kapplication.h>
-
 #include "viewtree.h"
 #include "viewtreeitem.h"
 #include "preferences.h"
@@ -29,6 +16,49 @@
 #include "server.h"
 #include "channel.h"
 #include "ircview.h"
+#include "konsolepanel.h"
+
+#include <qheader.h>
+#include <qdragobject.h>
+#include <qlistview.h>
+#include <qptrlist.h>
+#include <qpoint.h>
+#include <qpainter.h>
+#include <qtooltip.h>
+
+#include <kdebug.h>
+#include <klocale.h>
+#include <kglobalsettings.h>
+#include <kapplication.h>
+
+
+class ViewTree::ToolTip : public QToolTip
+{
+    public:
+        ToolTip(QWidget *parent, KListView *viewTree);
+        virtual ~ToolTip() {}
+
+    protected:
+        virtual void maybeTip(const QPoint &pos);
+
+    private:
+        KListView* viewTree;
+};
+
+ViewTree::ToolTip::ToolTip(QWidget *parent, KListView *viewTree)
+    : QToolTip(parent), viewTree(viewTree)
+{
+}
+
+void ViewTree::ToolTip::maybeTip (const QPoint &pos)
+{
+    if (!parentWidget() || !viewTree) return;
+
+    ViewTreeItem* view = static_cast<ViewTreeItem*>(viewTree->itemAt(pos));
+
+    if (view && view->isTruncated()) tip(viewTree->itemRect(view), view->getName());
+}
+
 
 ViewTree::ViewTree(QWidget *parent)
     : KListView(parent)
@@ -48,6 +78,9 @@ ViewTree::ViewTree(QWidget *parent)
     setAcceptDrops(true);
     setDropVisualizer(true);
 
+    setShowToolTips(false);
+    m_toolTip = new ViewTree::ToolTip(viewport(), this);
+
     // Controls whether or not to select the first view added
     // to the tree. Don't do so by default; only when told to
     // by the ViewContainer.
@@ -59,6 +92,8 @@ ViewTree::ViewTree(QWidget *parent)
     m_closeButtonItem = 0;
     m_enableCloseButtonTimer = new QTimer(this);
 
+    m_middleClickItem = 0;
+
     connect(m_enableCloseButtonTimer, SIGNAL(timeout()), SLOT(enableCloseButton()));
     connect(this, SIGNAL(selectionChanged(QListViewItem*)), SLOT(announceSelection(QListViewItem*)));
     connect(this, SIGNAL(aboutToMove()), SLOT(slotAboutToMoveView()));
@@ -67,6 +102,8 @@ ViewTree::ViewTree(QWidget *parent)
 
 ViewTree::~ViewTree()
 {
+    delete m_toolTip;
+    m_toolTip = 0;
     emit setViewTreeShown(false);
 }
 
@@ -90,6 +127,7 @@ void ViewTree::updateAppearance()
         fg = KGlobalSettings::textColor();
     }
 
+    setPalette(KApplication::palette());
     setPaletteForegroundColor(fg);
     setPaletteBackgroundColor(bg);
 }
@@ -513,6 +551,7 @@ void ViewTree::contentsMousePressEvent(QMouseEvent* e)
             m_pressedAboveCloseButton = false;
             KListView::contentsMousePressEvent(e);
         }
+        m_middleClickItem = (Preferences::middleClickClose() && e->button() == MidButton) ? item : 0;
     }
 }
 
@@ -524,13 +563,22 @@ void ViewTree::contentsMouseReleaseEvent(QMouseEvent* e)
     if (!item && e->button() == RightButton)
         return;
 
-    if (Preferences::closeButtons() && item)
+    if (item)
     {
-        if (e->button() == LeftButton && isAboveIcon(vp, item)
-            && m_pressedAboveCloseButton && item->getCloseButtonEnabled())
+        if (Preferences::closeButtons() && e->button() == LeftButton
+            && isAboveIcon(vp, item) && m_pressedAboveCloseButton
+            && item->getCloseButtonEnabled())
         {
             emit closeView(item->getView());
         }
+
+        if (Preferences::middleClickClose() && e->button() == MidButton
+            && item == m_middleClickItem)
+        {
+            emit closeView(item->getView());
+
+            m_middleClickItem = 0;
+        }
     }
     else
         KListView::contentsMouseReleaseEvent(e);
@@ -544,6 +592,9 @@ void ViewTree::contentsMouseMoveEvent(QMouseEvent* e)
     if (item && item->isSeparator())
         return;
 
+    // Cancel middle-click close.
+    if (item != m_middleClickItem) m_middleClickItem = 0;
+
     // Allow dragging only with the middle mouse button, just
     // like for the tab bar.
     if ((e->state() & MidButton) == MidButton)
@@ -621,16 +672,24 @@ void ViewTree::keyPressEvent(QKeyEvent* e)
     else
     {
         ViewTreeItem* item = static_cast<ViewTreeItem*>(selectedItem());
-        if (item && item->getView()->isInsertSupported())
+        if (item && item->getView() && item->getView()->isInsertSupported())
         {
             KApplication::sendEvent(item->getView()->getTextView(), e);
             item->getView()->adjustFocus();
         }
+        else if (item && item->getView() && item->getView()->getType() == ChatWindow::Konsole)
+        {
+            KonsolePanel* panel = static_cast<KonsolePanel*>(item->getView());
+            KApplication::sendEvent(panel->getWidget(), e);
+            item->getView()->adjustFocus();
+        }
     }
 }
 
 void ViewTree::selectUpper(bool wrap)
 {
+    if (!selectedItem()) return;
+
     ViewTreeItem* itemAbove = static_cast<ViewTreeItem*>(selectedItem()->itemAbove());
 
     if (itemAbove)
@@ -650,6 +709,8 @@ void ViewTree::selectUpper(bool wrap)
 
 void ViewTree::selectLower(bool wrap)
 {
+    if (!selectedItem()) return;
+
     ViewTreeItem* itemBelow = static_cast<ViewTreeItem*>(selectedItem()->itemBelow());
 
     if (itemBelow)
@@ -719,19 +780,20 @@ void ViewTree::findDrop(const QPoint &pos, QListViewItem *&parent, QListViewItem
             if (itemAbove->sortLast())
             {
                 after = m_separator->itemAbove();
-                parent = after ? after->parent() : 0L;
+                after = (!after || after->depth() == 0) ? after : after->parent();
+                parent = 0L;
                 return;
             }
             else if (above->depth() == dragItem->depth())
             {
                 after = above;
-                parent = after->parent();
+                parent = 0L;
                 return;
             }
             else
             {
                 after = above->parent();
-                parent = after ? after->parent() : 0L;
+                parent = 0L;
                 return;
             }
         }
