@@ -6,18 +6,20 @@
 */
 
 /*
-  copyright: (C) 2006 by Eike Hein
-  email:     sho@eikehein.com
+  Copyright (C) 2006 Eike Hein <hein@kde.org>
 */
 
 #ifndef VIEWTREEITEM_H
 #define VIEWTREEITEM_H
 
+#include "chatwindow.h"
+
 #include <qobject.h>
 #include <qlistview.h>
+#include <qtooltip.h>
 #include <qguardedptr.h>
+#include <qpixmap.h>
 
-#include "chatwindow.h"
 
 class ChatWindow;
 class Images;
@@ -38,6 +40,7 @@ class ViewTreeItem : public QListViewItem
 
         void setName(const QString& name);
         QString getName() const;
+        bool isTruncated() const;
 
         void setView(ChatWindow* view);
         ChatWindow* getView() const;
@@ -68,13 +71,13 @@ class ViewTreeItem : public QListViewItem
     private:
         uint m_sortIndex;
         static int s_availableSortIndex;
-        QString m_name;
         QGuardedPtr<ChatWindow> m_view;
         ChatWindow::WindowType m_viewType;
         QColor m_color;
 
         bool m_isSeparator;
         bool m_isHighlighted;
+        bool m_isTruncated;
         bool m_customColorSet;
 
         Images* images;
